# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from autopilot import platform

from lomiriuitoolkit import lomiri_scenarios
from lomiriuitoolkit.tests import gallery


class ScrollBarTestCase(gallery.GalleryTestCase):

    scenarios = lomiri_scenarios.get_device_simulation_scenarios()

    def setUp(self):
        super().setUp()
        self.open_page('navigationElement')
        self.scrollview = self.main_view.wait_select_single(
            'ScrollView', objectName="TemplateScrollView")
        self.scrollbar = self.scrollview.select_single(
            objectName="verticalScrollbar")

    def move_mouse_to_thumb(self):
        # TODO we need a helper to move the interactive thumb.
        # --elopio 2014-05-06
        thumb = self.get_mouse_thumb()
        self.pointing_device.move_to_object(thumb)

    def get_mouse_thumb(self):
        return self.scrollbar.select_single(
            objectName='interactiveScrollbarThumb')

    def test_drag_thumb_down_must_make_bottom_visible(self):
        if platform.model() != 'Desktop':
            self.skipTest(
                'The interactive thumb is activated by the move of a mouse')

        bottom_section = self.main_view.select_single(className='PageStack')
        flickable = self.main_view.select_single(
            'QQuickFlickable', objectName='TemplateFlickable')
        self.assertEqual(flickable.is_child_visible(bottom_section), False)

        self.move_mouse_to_thumb()
        mouse_thumb = self.get_mouse_thumb()
        x, y, width, height = mouse_thumb.globalRect
        start_x = stop_x = x + (width // 2)
        start_y = y + (height // 2)
        stop_y = self.main_view.y + self.main_view.height

        self.pointing_device.drag(start_x, start_y, stop_x, stop_y)

        self.assertEqual(flickable.is_child_visible(bottom_section), True)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging
import time

from autopilot import (
    input,
    logging as autopilot_logging,
    platform
)
from autopilot.introspection import dbus

from lomiriuitoolkit._custom_proxy_objects import _common, _flickable

logger = logging.getLogger(__name__)


class QQuickListView(_flickable.QQuickFlickable):

    @autopilot_logging.log_action(logger.info)
    def click_element(self, object_name, direction=None):
        """Click an element from the list.

        It swipes the element into view if it's center is not visible.

        :parameter object_name: The objectName property of the element to
            click.
        :parameter direction: The direction where the element is, it can be
            either 'above' or 'below'. Default value is None, which means we
            don't know where the object is and we will need to search the full
            list.

        """
        try:
            element = self.select_single(objectName=object_name)
        except dbus.StateNotFoundError:
            # The element might be on a part of the list that hasn't been
            # created yet. We have to search for it scrolling the entire list.
            element = self._find_element(object_name, direction)
        self.swipe_child_into_view(element)
        self.pointing_device.click_object(element)
        return element

    @autopilot_logging.log_action(logger.info)
    def _find_element(self, object_name, direction=None):
        if direction is None:
            # We don't know where the object is so we start looking for it from
            # the top or left, depending on the ListView orientation.
            if self.orientation == 2:
                # 2 == ListView.Vertical
                self.swipe_to_top()
                direction = 'below'
            else:
                # orientation == 1 == ListView.Horizontal
                self.swipe_to_leftmost()
                direction = 'right'

        if direction == 'below':
            def fail_condition():
                return self.atYEnd
            swipe_method = self.swipe_to_show_more_below
        elif direction == 'above':
            def fail_condition():
                return self.atYBeginning
            swipe_method = self.swipe_to_show_more_above
        elif direction == 'left':
            def fail_condition():
                return self.atXBeginning
            swipe_method = self.swipe_to_show_more_left
        elif direction == 'right':
            def fail_condition():
                return self.atXEnd
            swipe_method = self.swipe_to_show_more_right
        else:
            raise _common.ToolkitException(
                'Invalid direction: {}'.format(direction))

        containers = self._get_containers()
        while not fail_condition():
            try:
                return self.select_single(objectName=object_name)
            except dbus.StateNotFoundError:
                swipe_method(containers)
        else:
            # We have reached the start or the end of the list.
            try:
                return self.select_single(objectName=object_name)
            except dbus.StateNotFoundError:
                raise _common.ToolkitException(
                    'List element with objectName "{}" not found.'.format(
                        object_name))

    def _is_element_cached(self, object_name):
        """Return the object with the given object name if the element
           is cached, or None if the element is not cached.
        """
        try:
            child = self.select_single(objectName=object_name)
        except dbus.StateNotFoundError:
            return None

        return child

    def _is_element_visible(self, object_name):
        """Return True if the center of element with the given object name
           is visible and False otherwise.
        """
        child = self._is_element_cached(object_name)
        if not child:
            return False

        containers = self._get_containers()
        return self._is_child_visible(child, containers)

    @autopilot_logging.log_action(logger.info)
    def enable_select_mode(self):
        """Default implementation to enable select mode. Performs a long tap
           over the first list item in the ListView. The delegates must be
           the new ListItem components.
        """
        self.swipe_to_top()
        first_item = self._get_first_item()
        self.pointing_device.click_object(first_item, press_duration=2)
        try:
            self.wait_select_single('QQuickItem',
                                    objectName='selection_panel0')
        except dbus.StateNotFoundError:
            raise _common.ToolkitException(
                'ListView delegate is not a ListItem or not in selectMode')

    def _get_first_item(self):
        """Returns the first item from the ListView."""
        items = self.get_children_by_type('QQuickItem')[0].get_children()
        items = sorted(items, key=lambda item: item.globalRect.y)
        return items[0]

    @autopilot_logging.log_action(logger.info)
    def drag_item(self, from_index, to_index):
        if platform.model() != 'Desktop':
            raise NotImplementedError(
                'Drag does not work on the phone because of bug #1266601')

        self._enable_drag_mode()

        both_items_visible = (
            self._is_drag_panel_visible(from_index) and
            self._is_drag_panel_visible(to_index))
        if both_items_visible:
            self._drag_both_items_visible(from_index, to_index)
        else:
            self._drag_item_with_pagination(from_index, to_index)
        # wait 1 second till all animations complete
        time.sleep(1)

    @autopilot_logging.log_action(logger.debug)
    def _enable_drag_mode(self):
        self.swipe_to_top()
        first_item = self._get_first_item()
        self.pointing_device.click_object(first_item, press_duration=2)
        self._get_drag_panel(wait=True)

    def _is_drag_panel_visible(self, index):
        try:
            drag_panel = self._get_drag_panel(index)
        except:
            return False
        else:
            return self.is_child_visible(drag_panel)

    def _get_drag_panel(self, index=0, wait=False):
        select_method = self.wait_select_single if wait else self.select_single
        try:
            return select_method(
                'QQuickItem', objectName='drag_panel{}'.format(index))
        except ValueError:
            # While we are dragging, each panel item is duplicated.
            # A ValueError will be raised in this scenario. We can take
            # any of the returned elements, as they have the same position.
            return self.select_many(
                'QQuickItem', objectName='drag_panel{}'.format(index))[0]

    def _drag_both_items_visible(self, from_index, to_index):
        from_drag_handler = self._get_drag_panel(from_index)
        to_drag_handler = self._get_drag_panel(to_index)
        start_x, start_y = input.get_center_point(from_drag_handler)
        stop_x, stop_y = input.get_center_point(to_drag_handler)
        self.pointing_device.drag(start_x, start_y, stop_x, stop_y)

    def _drag_item_with_pagination(self, from_index, to_index):
        # the from_index might be invisible
        from_item = self._find_element('listitem{}'.format(from_index))
        from_item.swipe_into_view()
        from_drag_panel = self._get_drag_panel(from_index)
        containers = self._get_containers()
        if from_index < to_index:
            self._drag_downwards(from_drag_panel, to_index, containers)
        else:
            self._drag_upwards(from_drag_panel, to_index, containers)

    def _drag_downwards(self, drag_panel, to_index, containers):
        visible_bottom = _flickable._get_visible_container_bottom(
            containers)
        start_x, start_y = input.get_center_point(drag_panel)

        self.pointing_device.move(start_x, start_y)
        self.pointing_device.press()
        stop_x = start_x
        self.pointing_device.move(stop_x, visible_bottom)
        to_drag_panel = self._get_drag_panel(to_index, wait=True)
        while not self.is_child_visible(to_drag_panel):
            pass
        # stop moving
        h = to_drag_panel.height / 2
        self.pointing_device.move(stop_x, self.pointing_device.y - h)
        # move under the item with the to_index
        to_drag_panel = self._get_drag_panel(to_index - 1)
        stop_y = (
            to_drag_panel.globalRect.y +
            to_drag_panel.globalRect.height)
        self.pointing_device.move(stop_x, stop_y)
        self.pointing_device.release()

    def _drag_upwards(self, handler, to_index, containers):
        visible_top = _flickable._get_visible_container_top(
            containers)
        start_x, start_y = input.get_center_point(handler)

        self.pointing_device.move(start_x, start_y)
        self.pointing_device.press()
        stop_x = start_x
        # Header alters topMargin, therefore drag only till that edge
        self.pointing_device.move(stop_x, visible_top + self.topMargin)
        to_drag_panel = self._get_drag_panel(to_index, wait=True)
        while not self.is_child_visible(to_drag_panel):
            pass
        # stop moving
        h = to_drag_panel.height / 2
        self.pointing_device.move(stop_x, self.pointing_device.y + h)
        # move after the item with the to_index
        to_drag_panel = self._get_drag_panel(to_index)
        stop_y = (
            to_drag_panel.globalRect.y +
            to_drag_panel.globalRect.height)
        self.pointing_device.move(stop_x, stop_y)
        self.pointing_device.release()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013-2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from autopilot.introspection import dbus

import os
import lomiriuitoolkit
from lomiriuitoolkit import popups, tests


class WindowTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_popups.WindowTestCase.qml')

    def _open_dialog(self):
        open_button = self.main_view.select_single(
            'Button', objectName='open_dialog')
        self.pointing_device.click_object(open_button)
        return self.main_view.wait_select_single(
            popups.Dialog, objectName='test_dialog')

    def test_can_open_dialog(self):
        popover = self._open_dialog()
        self.assertIsInstance(popover, popups.Dialog)


class ActionSelectionPopoverTestCase(tests.QMLFileAppTestCase):

    # FIXME: Currently we are only testing UITK 1.0 Popups,
    #   we need to test 1.3 as well. See bug 1612582.
    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_popups.ActionSelectionPopoverTestCase.qml')

    def test_custom_proxy_object(self):
        popover = self.main_view.get_action_selection_popover(
            'test_actions_popover')
        self.assertIsInstance(popover, popups.ActionSelectionPopover)

    def test_click_button_by_label(self):
        label = self.app.select_single('Label', objectName='clicked_label')
        self.assertNotEqual(label.text, 'Button clicked.')
        self._open_popover()
        popover = self.main_view.get_action_selection_popover(
            'test_actions_popover')
        popover.click_button_by_text('Action one')
        self.assertEqual(label.text, 'Button clicked.')

    def test_click_button_by_object_name(self):
        label = self.app.select_single('Label', objectName='clicked_label')
        self.assertNotEqual(label.text, 'Button clicked.')
        self._open_popover()
        popover = self.main_view.get_action_selection_popover(
            'test_actions_popover')
        popover.click_action_button('actionOne')
        self.assertEqual(label.text, 'Button clicked.')

    def test_click_unexisting_button_by_object_name(self):
        self._open_popover()
        popover = self.main_view.get_action_selection_popover(
            'test_actions_popover')
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            popover.click_action_button, 'actionTwo')
        self.assertEqual(
            str(error),
            'Button for action with objectName "actionTwo" not found.')

    def test_click_disabled_button_by_object_name(self):
        self._open_popover()
        popover = self.main_view.get_action_selection_popover(
            'test_actions_popover')
        # Disabled actions are not shown in ActionSelectionPopover
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            popover.click_action_button, 'actionDisabled')
        self.assertEqual(
            str(error),
            'Button for action with objectName "actionDisabled" not visible.')

    def test_click_hidden_button_by_object_name(self):
        self._open_popover()
        popover = self.main_view.get_action_selection_popover(
            'test_actions_popover')
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            popover.click_action_button, 'actionHidden')
        self.assertEqual(
            str(error),
            'Button for action with objectName "actionHidden" not visible.')

    def _open_popover(self):
        open_button = self.main_view.select_single(
            'Button', objectName='open_popover')
        self.pointing_device.click_object(open_button)

    def test_click_unexisting_button(self):
        self._open_popover()
        popover = self.main_view.get_action_selection_popover(
            'test_actions_popover')
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, popover.click_button_by_text,
            'unexisting')
        self.assertEqual(
            str(error), 'Button with text "unexisting" not found.')

    def test_click_button_with_closed_popover(self):
        popover = self.main_view.get_action_selection_popover(
            'test_actions_popover')
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, popover.click_button_by_text,
            'Action one')
        self.assertEqual(
            str(error), 'The popover is not open.')


class ComposerSheetTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_popups.ComposerSheetTestCase.qml')

    def setUp(self):
        super().setUp()
        self.label = self.main_view.select_single(
            'Label', objectName='actionLabel')
        self.assertEqual(self.label.text, 'No action taken.')
        self._open_composer_sheet()
        self.composer_sheet = self._select_composer_sheet()

    def _open_composer_sheet(self):
        button = self.main_view.select_single(
            'Button', objectName='openComposerSheetButton')
        self.pointing_device.click_object(button)

    def _select_composer_sheet(self):
        return self.main_view.select_single(
            popups.ComposerSheet, objectName='testComposerSheet')

    def test_select_composer_sheet_custom_proxy_object(self):
        self.assertIsInstance(
            self.composer_sheet, popups.ComposerSheet)

    def test_confirm_composer_sheet(self):
        self.composer_sheet.confirm()
        self.assertEqual(self.label.text, 'Confirm selected.')
        self._assert_composer_sheet_is_closed()

    def _assert_composer_sheet_is_closed(self):
        self.assertRaises(
            dbus.StateNotFoundError, self._select_composer_sheet)

    def test_cancel_composer_sheet(self):
        self.composer_sheet.cancel()
        self.assertEqual(self.label.text, 'Cancel selected.')
        self._assert_composer_sheet_is_closed()



#!/usr/bin/python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# lomiri-ui-toolkit Autopilot Test Suite
# Copyright (C) 2012, 2013, 2015 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from setuptools import setup

setup(
    name='lomiriuitoolkit',
    version='1.3.5906',
    description='lomiri-ui-toolkit autopilot tests.',
    url='https://gitlab.com/ubports/development/core/lomiri-ui-tookit',
    license='LGPLv3',
    packages=[ 'lomiriuitoolkit' ],
)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013-2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
from unittest import mock

import lomiriuitoolkit
from lomiriuitoolkit import tests


class MainView10TestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_main_view.MainView10TestCase.qml')

    def test_main_view_custom_proxy_object(self):
        self.assertIsInstance(self.main_view, lomiriuitoolkit.MainView)

    def test_get_header_without_header(self):
        header = self.main_view.get_header()
        self.assertFalse(header.visible)

    def test_toolbar_custom_proxy_object(self):
        toolbar = self.main_view.get_toolbar()
        self.assertIsInstance(toolbar, lomiriuitoolkit.Toolbar)

    def test_open_toolbar(self):
        with mock.patch.object(lomiriuitoolkit.Toolbar, 'open') as mock_open:
            self.main_view.open_toolbar()

        mock_open.assert_called_once_with()

    def test_close_toolbar(self):
        with mock.patch.object(lomiriuitoolkit.Toolbar, 'close') as mock_close:
            self.main_view.close_toolbar()

        mock_close.assert_called_once_with()

    def test_open_toolbar_returns_the_toolbar(self):
        toolbar = self.main_view.open_toolbar()
        self.assertIsInstance(toolbar, lomiriuitoolkit.Toolbar)

    def test_get_tabs_without_tabs(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, self.main_view.get_tabs)
        self.assertEqual(
            str(error), 'The MainView has no Tabs.')

    def test_switch_to_next_tab_without_tabs(self):
        header = self.main_view.get_header()
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            header.switch_to_next_tab)
        self.assertEqual(
            str(error), 'The MainView has no Tabs.')


class MainView12TestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_main_view.MainView12TestCase.qml')

    def test_main_view_custom_proxy_object(self):
        self.assertIsInstance(self.main_view, lomiriuitoolkit.MainView)

    def test_get_header_without_header(self):
        header = self.main_view.get_header()
        self.assertFalse(header.visible)

    def test_get_tabs_without_tabs(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, self.main_view.get_tabs)
        self.assertEqual(
            str(error), 'The MainView has no Tabs.')

    def test_switch_to_next_tab_without_tabs(self):
        header = self.main_view.get_header()
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            header.switch_to_next_tab)
        self.assertEqual(
            str(error), 'The MainView has no Tabs.')

    def test_get_toolbar_without_toolbar(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, self.main_view.get_toolbar)
        self.assertEqual(
            str(error), 'The MainView has no Toolbar.')

    def test_open_toolbar_without_toolbar(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, self.main_view.open_toolbar)
        self.assertEqual(
            str(error), 'The MainView has no Toolbar.')

    def test_close_toolbar_without_toolbar(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, self.main_view.close_toolbar)
        self.assertEqual(
            str(error), 'The MainView has no Toolbar.')


class GoBackTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    toolbar_test_qml_file_path = os.path.join(
        dir_path, 'test_main_view.GoBackTestCase.back_in_toolbar.qml')
    header_test_qml_file_path = os.path.join(
        dir_path, 'test_main_view.GoBackTestCase.back_in_header.qml')

    scenarios = [
        ('back_in_toolbar', dict(
            test_qml_file_path=toolbar_test_qml_file_path)),
        ('back_in_header', dict(
            test_qml_file_path=header_test_qml_file_path))
    ]

    def setUp(self):
        super().setUp()
        self.header = self.main_view.get_header()
        self.assertEqual(self.header.title, 'Page 0')

    def test_open_page(self):
        self._go_to_page1()
        self.assertEqual(self.header.title, 'Page 1')

    def _go_to_page1(self):
        button = self.main_view.select_single(
            'Button',
            objectName='go_to_page1')
        self.pointing_device.click_object(button)
        self.header.wait_for_animation()

    def test_go_back(self):
        self._go_to_page1()
        self.main_view.go_back()
        self.assertEqual(self.header.title, 'Page 0')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the Button Qml component."""

from autopilot.matchers import Eventually
from textwrap import dedent
from testtools.matchers import Is, Not, Equals
from testtools import skip
import os
from tavastia.tests import TavastiaTestCase

class EnabledButtonTests(TavastiaTestCase):
    """Tests for an enabled Button component."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"EnabledButtonTests")

    def test_can_select_button(self):
        """Must be able to select the Qml button component."""

        btn = self.app.select_single('Button')
        self.assertThat(btn, Not(Is(None)))


    def test_clicked_signal_emitted(self):
        """Clicking the button component must emit the clicked() signal."""
        
        btn = self.app.select_single('Button')
        signal = btn.watch_signal('clicked(QVariant)')

        self.mouse.move_to_object(btn)
        self.mouse.click()

        self.assertThat(signal.was_emitted, Equals(True))
        self.assertThat(signal.num_emissions, Equals(1))


    def test_entered_signal_emitted(self):
        """The hoveredChanged() signal must be emitted when the mouse hovers over
        the button. Then the hovered property should be true"""

        btn = self.app.select_single('Button')
        signal = btn.watch_signal('hoveredChanged()')

        self.mouse.move_to_object(btn)

        self.assertThat(signal.was_emitted, Equals(True))
        self.assertThat(signal.num_emissions, Equals(1))
        self.assertThat(btn.hovered, Eventually(Equals(True)))


    def test_exited_signal_emitted(self):
        """The hoveredChanged() signal must be emitted when the mouse is hovering
        over hovers the button, and is moved out. Then the hovered property should be 
        false"""

        btn = self.app.select_single('Button')

        # position mouse over button before starting to watch for signals
        self.mouse.move_to_object(btn)
        signal = btn.watch_signal('hoveredChanged()')

        # assuming the WM will never put the window over 0,0:
        self.mouse.move(0,0)

        self.assertThat(signal.was_emitted, Equals(True))
        self.assertThat(signal.num_emissions, Equals(1))
        self.assertThat(btn.hovered, Eventually(Equals(False)))


    def test_can_press_button(self):
        """Test that when we click and hold the button down it's pressed property is set."""

        btn = self.app.select_single('Button')

        self.mouse.move_to_object(btn)
        self.mouse.press()
        self.addCleanup(self.mouse.release)

        self.assertThat(btn.pressed, Eventually(Equals(True)))


class DisabledButtonTests(TavastiaTestCase):
    """Tests for a disabled button component."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"DisabledButtonTests")

    def test_can_select_button(self):
        """Must be able to select the Qml button component."""

        btn = self.app.select_single('Button')
        self.assertThat(btn, Not(Is(None)))


    def test_button_is_disabled(self):
        """Test that the button really is disabled."""

        btn = self.app.select_single('Button')

        self.assertThat(btn.enabled, Equals(False))


    def test_clicked_signal_not_emitted(self):
        """Clicking a disabled button must not emit the clicked() signal."""

        btn = self.app.select_single('Button')
        signal = btn.watch_signal('clicked(QVariant)')

        self.mouse.move_to_object(btn)
        self.mouse.click()

        self.assertThat(signal.was_emitted, Equals(False))
        self.assertThat(signal.num_emissions, Equals(0))

class ButtonColorTests(TavastiaTestCase):
    """Tests for the button color properties."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"ButtonColorTests")

    def test_can_select_button(self):
        """Must be able to select the Qml button component."""

        btn = self.app.select_single('Button')
        self.assertThat(btn, Not(Is(None)))


    def test_button_has_correct_color(self):
        """Button component must have correct color."""

        btn = self.app.select_single('Button')
        self.assertThat(btn.color, Eventually(Equals([255,255,0,255])))



######################
# Test commented out due to bug in autopilot-qt5, which has no support for 'property alias'
# https://bugs.launchpad.net/autopilot-qt/+bug/1073265
# 
#class ButtonIconTests(TavastiaTestCase):
#    """Tests for the button icon properties"""
#
#    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"ButtonIconTests")
#
#    def test_can_select_button(self):
#        """Must be able to select the Qml button component."""
#
#        btn = self.app.select_single('Button')
#        self.assertThat(btn, Not(Is(None)))
#
#
#    def test_can_set_icon(self):
#        """Must be able to set icon for the Button"""
#
#        btn = self.app.select_single('Button')
#        dir(btn)
#        self.assertThat(btn.iconSource, Equals("call_icon.png"))
#
#
#    def test_can_set_icon_position(self):
#        """Must be able to set icon position for the Button"""
#
#        btn = self.app.select_single('Button')
#        self.assertThat(btn.iconPosition, Equals("right"))





# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013-2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from unittest import mock

import logging
import os

import fixtures
import testtools
from testtools.matchers import Contains

import lomiriuitoolkit
from lomiriuitoolkit import listitems, tests


class ListItemsTestCase(testtools.TestCase):

    def test_supported_class(self):
        self.assertTrue(issubclass(
            listitems.Base, listitems.Empty))
        self.assertTrue(issubclass(
            listitems.Expandable, listitems.Empty))
        self.assertTrue(issubclass(
            listitems.ItemSelector, listitems.Empty))
        self.assertTrue(issubclass(
            listitems.Standard, listitems.Empty))
        self.assertTrue(issubclass(
            listitems.SingleControl, listitems.Empty))
        self.assertTrue(issubclass(
            listitems.MultiValue, listitems.Base))
        self.assertTrue(issubclass(
            listitems.SingleValue, listitems.Base))
        self.assertTrue(issubclass(
            listitems.Subtitled, listitems.Base))


class SwipeToDeleteTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_listitems.SwipeToDeleteTestCase.qml')

    def setUp(self):
        super().setUp()
        self._item = self.main_view.select_single(
            listitems.Standard, objectName='listitem_standard')
        self.assertTrue(self._item.exists())

    def test_standard_custom_proxy_object(self):
        self.assertIsInstance(self._item, listitems.Standard)

    def test_swipe_item(self):
        self._item.swipe_to_delete()
        self.assertTrue(self._item.waitingConfirmationForRemoval)

    def test_swipe_item_to_right(self):
        self._item.swipe_to_delete('right')
        self.assertTrue(self._item.waitingConfirmationForRemoval)

    def test_swipe_item_to_left_must_raise_exception(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            self._item.swipe_to_delete,
            'left')
        self.assertEqual(
            str(error),
            'Only swiping to the right will cause the item to be deleted.')

    def test_swipe_item_with_direction_must_log_deprecation_warning(self):
        fake_logger = fixtures.FakeLogger(level=logging.WARNING)
        self.useFixture(fake_logger)

        self._item.swipe_to_delete(direction='right')
        self.assertThat(
            fake_logger.output,
            Contains(
                'The direction argument is deprecated. Now the Lomiri SDK '
                'only deletes list items when swiping from left to right. '
                'Call swipe_to_delete without arguments.'))

    def test_swipe_item_to_wrong_direction(self):
        self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            self._item.swipe_to_delete, 'up')

    def test_delete_item_moving_right(self):
        self._item.swipe_to_delete('right')
        self._item.confirm_removal()
        self.assertFalse(self._item.exists())

    def test_delete_non_removable_item(self):
        self._item = self.main_view.select_single(
            listitems.Empty, objectName='listitem_empty')
        self.assertRaises(
            lomiriuitoolkit.ToolkitException, self._item.swipe_to_delete)

    def test_confirm_removal_when_item_was_not_swiped(self):
        self.assertRaises(
            lomiriuitoolkit.ToolkitException, self._item.confirm_removal)

    def test_delete_item_without_confirm(self):
        item = self.main_view.select_single(
            listitems.Standard, objectName='listitem_without_confirm')
        item.swipe_to_delete()
        self.assertFalse(item.exists())

    def test_delete_item_with_confirmation_that_will_be_destroyed(self):
        item = self.main_view.select_single(
            listitems.Standard,
            objectName='listitem_destroyed_on_remove_with_confirm')
        item.swipe_to_delete()
        item.confirm_removal()
        self.assertFalse(item.exists())

    def test_delete_item_without_confirmation_that_will_be_destroyed(self):
        item = self.main_view.select_single(
            listitems.Standard,
            objectName='listitem_destroyed_on_remove_without_confirm')
        item.swipe_to_delete()
        self.assertFalse(item.exists())


class ExpandableTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_listitems.ExpandableTestCase.qml')

    def setUp(self):
        super().setUp()
        self.test_expandable = self.main_view.select_single(
            listitems.Expandable, objectName='expandable0')

    def test_expand_item(self):
        self.test_expandable.expand()
        self.assertTrue(self.test_expandable.expanded)

    def test_expand_already_expanded_item_must_do_nothing(self):
        self.test_expandable.expand()

        # Replace the pointer with None, so we make sure it's not being called.
        with mock.patch.object(self.test_expandable, 'pointing_device', None):
            self.test_expandable.expand()

    def test_collapse_item(self):
        self.test_expandable.expand()

        self.test_expandable.collapse()
        self.assertFalse(self.test_expandable.expanded)

    def test_collapse_already_collapsed_item_must_do_nothing(self):
        self.test_expandable.collapse()

        # Replace the pointer with None, so we make sure it's not being called.
        with mock.patch.object(self.test_expandable, 'pointing_device', None):
            self.test_expandable.collapse()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import logging as autopilot_logging
from lomiriuitoolkit._custom_proxy_objects import _common


logger = logging.getLogger(__name__)


class TabBar(_common.LomiriUIToolkitCustomProxyObjectBase):
    """TabBar Autopilot custom proxy object."""

    @autopilot_logging.log_action(logger.info)
    def switch_to_next_tab(self):
        """Open the next tab."""
        self._activate_tab_bar()
        logger.debug('Click the next tab bar button.')
        self.pointing_device.click_object(self._get_next_tab_button())

    def _activate_tab_bar(self):
        # First move to the tab bar to avoid timing issues when we find it in
        # selection mode but it's deselected while we move to it.
        self.pointing_device.move_to_object(self)
        if self.selectionMode:
            logger.debug('Already in selection mode.')
        else:
            # Click the tab bar to switch to selection mode.
            logger.debug('Click the tab bar to enable selection mode.')
            self.pointing_device.click_object(self)

    def _get_next_tab_button(self):
        current_index = self._get_selected_button_index()
        next_index = (current_index + 1) % self._get_number_of_tab_buttons()
        return self._get_tab_button(next_index)

    def _get_selected_button_index(self):
        return self.select_single('QQuickPathView').selectedButtonIndex

    def _get_number_of_tab_buttons(self):
        return len(self._get_tab_buttons())

    def _get_tab_buttons(self):
        # TabBar is only used before 1.3, when there was no UCAbstractButton
        return self.select_many('AbstractButton')

    def _get_tab_button(self, index):
        buttons = self._get_tab_buttons()
        for button in buttons:
            if button.buttonIndex == index:
                return button
        raise _common.ToolkitException(
            'There is no tab button with index {0}.'.format(index))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests for the deprecated lomiriuitoolkit.emulators module."""

import imp
import logging

import fixtures
import testscenarios
import testtools
from testtools.matchers import Contains

import lomiriuitoolkit
from lomiriuitoolkit import emulators, listitems, popups


class DeprecatedSymbolsTestCase(testscenarios.TestWithScenarios):

    symbols_retaining_name = [
        'check_autopilot_version', 'get_keyboard', 'get_pointing_device',
        'CheckBox', 'Header', 'MainView', 'OptionSelector', 'QQuickFlickable',
        'QQuickListView', 'TabBar', 'Tabs', 'TextField', 'Toolbar',
    ]

    symbols_retaining_name_scenarios = [
        (symbol, dict(
            current_module=lomiriuitoolkit, current_symbol=symbol,
            deprecated_symbol=symbol))
        for symbol in symbols_retaining_name
    ]

    symbols_with_new_name = [
        (lomiriuitoolkit, 'ToolkitException', 'ToolkitEmulatorException'),
        (lomiriuitoolkit, 'LomiriUIToolkitCustomProxyObjectBase',
         'LomiriUIToolkitEmulatorBase'),
        (popups, 'ActionSelectionPopover', 'ActionSelectionPopover'),
        (popups, 'ComposerSheet', 'ComposerSheet'),
        (listitems, 'Base', 'Base'),
        (listitems, 'Empty', 'Empty'),
        (listitems, 'ItemSelector', 'ItemSelector'),
        (listitems, 'MultiValue', 'MultiValue'),
        (listitems, 'SingleControl', 'SingleControl'),
        (listitems, 'SingleValue', 'SingleValue'),
        (listitems, 'Standard', 'Standard'),
        (listitems, 'Subtitled', 'Subtitled')
    ]

    symbols_with_new_name_scenarios = [
        ('{0} to {1}'.format(old_name, new_name),
         dict(
             current_module=new_module, current_symbol=new_name,
             deprecated_symbol=old_name))
        for new_module, new_name, old_name in symbols_with_new_name]

    scenarios = (symbols_retaining_name_scenarios +
                 symbols_with_new_name_scenarios)

    def test_deprecated_symbol_must_be_the_same_as_current(self):
        self.assertEqual(
            getattr(self.current_module, self.current_symbol),
            getattr(emulators, self.deprecated_symbol))


class DeprecationWarningTestCase(testtools.TestCase):

    def reload_emulators(self):
        _, lomiriuitoolkit_path, _ = imp.find_module('lomiriuitoolkit')
        emulators_file, emulators_path, emulators_description = (
            imp.find_module('emulators', [lomiriuitoolkit_path]))
        imp.load_module(
            'emulators', emulators_file, emulators_path,
            emulators_description)

    def test_import_emulators_must_log_warning(self):
        fake_logger = fixtures.FakeLogger(level=logging.WARNING)
        self.useFixture(fake_logger)
        self.reload_emulators()
        self.assertThat(
            fake_logger.output,
            Contains(
                'The lomiriuitoolkit.emulators module is deprecated. Import '
                'the autopilot helpers from the top-level lomiriuitoolkit '
                'module.'))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the ActivityIndicator Qml component."""

from autopilot.matchers import Eventually
from textwrap import dedent
from testtools.matchers import Is, Not, Equals
from testtools import skip
import os
from tavastia.tests import TavastiaTestCase

class ActivityIndicatorTests(TavastiaTestCase):
    """Tests for ActivityIndicator component."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"ActivityIndicatorTests")

    def test_can_select_activityindicator(self):
        """Must be able to select the Qml ActivityIndicator component."""

        obj = self.app.select_single('ActivityIndicator')
        self.assertThat(obj, Not(Is(None)))
    
    def test_can_toggle_running(self):
        obj = self.app.select_single('ActivityIndicator')
        self.assertThat(obj, Not(Is(None)))

        self.assertThat(obj.running, Eventually(Equals(False)))
        
        btn = self.app.select_single('Button')

        self.mouse.move_to_object(btn)
        self.mouse.click()

        self.assertThat(obj.running, Eventually(Equals(True)))

        self.mouse.click()

        self.assertThat(obj.running, Eventually(Equals(False)))
        



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import testtools
from autopilot import input, platform

from lomiriuitoolkit import base


class AppTestCase(base.LomiriUIToolkitAppTestCase):
    """Empty test case to be used by other tests."""

    def _runTest(self):
        pass


class TestLomiriUIToolkitAppTestCase(testtools.TestCase):

    @testtools.skipIf(platform.model() != 'Desktop', 'Desktop only')
    def test_desktop_input_device_class(self):
        test = AppTestCase('_runTest')
        test.setUp()
        self.assertIs(test.input_device_class, input.Mouse)

    @testtools.skipIf(platform.model() == 'Desktop', 'Phablet only')
    def test_phablet_input_device_class(self):
        test = AppTestCase('_runTest')
        test.setUp()
        self.assertIs(test.input_device_class, input.Touch)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os

from unittest import mock

import lomiriuitoolkit
from lomiriuitoolkit import tests


class TabsTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    deprecated_tabbar_test_qml_file_path = os.path.join(
        dir_path, 'test_tabs.TabsTestCase.deprecated_TabBar.qml')
    new_header_test_qml_file_path = os.path.join(
        dir_path, 'test_tabs.TabsTestCase.new_header.qml')
    new_header_1_3_test_qml_file_path = os.path.join(
        dir_path, 'test_tabs.TabsTestCase.new_header.1.3.qml')

    scenarios = [
        ('deprecated TabBar',
         dict(test_qml_file_path=deprecated_tabbar_test_qml_file_path)),
        ('new header',
         dict(test_qml_file_path=new_header_test_qml_file_path)),
        ('new header 1.3',
         dict(test_qml_file_path=new_header_1_3_test_qml_file_path))
    ]

    def test_tabs_custom_proxy_object(self):
        self.assertIsInstance(self.main_view.get_tabs(), lomiriuitoolkit.Tabs)

    def test_get_current_tab(self):
        tabs = self.main_view.get_tabs()
        self.assertEqual(tabs.get_current_tab().title, 'Tab1')

    def test_switch_to_next_tab_from_header(self):
        header = self.main_view.get_header()
        header.switch_to_next_tab()
        current_tab = self.main_view.get_tabs().get_current_tab()
        self.assertEqual(current_tab.title, 'Tab2')

    def test_switch_to_next_tab_from_main_view(self):
        current_tab = self.main_view.switch_to_next_tab()
        self.assertEqual(current_tab.title, 'Tab2')

    def test_switch_to_next_tab_from_last(self):
        last_tab_index = self.main_view.get_tabs().get_number_of_tabs() - 1
        self.main_view.switch_to_tab_by_index(last_tab_index)
        current_tab = self.main_view.switch_to_next_tab()
        self.assertEqual(current_tab.title, 'Tab1')

    def test_switch_to_tab_by_index(self):
        current_tab = self.main_view.switch_to_tab_by_index(2)
        self.assertEqual(current_tab.title, 'Tab3')
        current_tab = self.main_view.switch_to_tab_by_index(1)
        self.assertEqual(current_tab.title, 'Tab2')
        current_tab = self.main_view.switch_to_tab_by_index(0)
        self.assertEqual(current_tab.title, 'Tab1')

    def test_switch_to_opened_tab_is_not_opened_again(self):
        with mock.patch.object(
                lomiriuitoolkit.AppHeader,
                'switch_to_next_tab') as mock_switch:
            current_tab = self.main_view.switch_to_tab_by_index(0)

        self.assertFalse(mock_switch.called)
        self.assertEqual(current_tab.title, 'Tab1')

    def test_get_number_of_tabs(self):
        tabs = self.main_view.get_tabs()
        self.assertEqual(tabs.get_number_of_tabs(), 3)

    def test_swith_to_tab_by_index_out_of_range(self):
        last_tab_index = self.main_view.get_tabs().get_number_of_tabs() - 1
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            self.main_view.switch_to_tab_by_index,
            last_tab_index + 1)
        self.assertEqual(str(error), 'Tab index out of range.')

    def test_switch_to_previous_tab_from_first(self):
        current_tab = self.main_view.switch_to_previous_tab()
        self.assertEqual(current_tab.title, 'Tab3')

    def test_switch_to_previous_tab_not_from_first(self):
        self.main_view.switch_to_tab_by_index(1)
        current_tab = self.main_view.switch_to_previous_tab()
        self.assertEqual(current_tab.title, 'Tab1')

    def test_switch_to_tab_by_object_name(self):
        current_tab = self.main_view.switch_to_tab('tab3')
        self.assertEqual(current_tab.title, 'Tab3')
        current_tab = self.main_view.switch_to_tab('tab2')
        self.assertEqual(current_tab.title, 'Tab2')
        current_tab = self.main_view.switch_to_tab('tab1')
        self.assertEqual(current_tab.title, 'Tab1')

    def test_switch_to_unexisting_tab(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, self.main_view.switch_to_tab,
            'unexisting')
        self.assertEqual(
            str(error), 'Tab with objectName "unexisting" not found.')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import lomiriuitoolkit
from lomiriuitoolkit import tests


class SectionsTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_sections.SectionsTestCase.qml')

    def setUp(self):
        super().setUp()
        self.sections = self.app.select_single(objectName='sections')
        self.scrollingSections =\
            self.app.select_single(objectName='scrolling_sections')
        self.label = self.app.select_single(objectName='label')
        self.scrollingLabel =\
            self.app.select_single(objectName='scrolling_label')
        self.assertEqual(self.label.text, 'Section 0 is selected.')
        self.assertEqual(self.scrollingLabel.text,
                         'Scrollable section 0 is selected.')

    def test_custom_proxy_object(self):
        self.assertIsInstance(self.sections, lomiriuitoolkit.Sections)
        self.assertTrue(self.sections.visible)
        self.assertIsInstance(self.scrollingSections, lomiriuitoolkit.Sections)
        self.assertTrue(self.scrollingSections.visible)

    def test_click_section_button(self):
        self.sections.click_section_button(2)
        self.assertEqual(self.label.text, 'Section 2 is selected.')

    def test_click_visible_scrolling_section_button(self):
        self.scrollingSections.click_section_button(2)
        self.assertEqual(self.scrollingLabel.text,
                         'Scrollable section 2 is selected.')

    def test_scroll_and_click_section_button(self):
        # scroll forward and click:
        self.scrollingSections.click_section_button(15)
        self.assertEqual(self.scrollingLabel.text,
                         'Scrollable section 15 is selected.')
        # scroll back and click:
        self.scrollingSections.click_section_button(1)
        self.assertEqual(self.scrollingLabel.text,
                         'Scrollable section 1 is selected.')

    def test_click_unexisting_section_button(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            self.sections.click_section_button, 3)
        self.assertEqual(
            str(error),
            'Button with section index 3 not found in Sections.')

        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            self.scrollingSections.click_section_button, 20)
        self.assertEqual(
            str(error),
            'Button with section index 20 not found in Sections.')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess
import tempfile

from unittest import mock
import testtools
from autopilot import (
    display,
    platform,
    testcase as autopilot_testcase
)
from autopilot.matchers import Eventually
from testtools.matchers import Contains, Equals, FileExists, Not

from lomiriuitoolkit import base, fixture_setup, tests


class FakeApplicationTestCase(testtools.TestCase):

    def assert_desktop_file_contents(
            self, desktop_file_contents, expected_contents_dict):
        desktop_file_lines = desktop_file_contents.splitlines()
        self.assertEqual('[Desktop Entry]', desktop_file_lines[0])

        contents_dict = dict(
            [line.split('=') for line in desktop_file_lines[1:]])
        self.assertDictEqual(expected_contents_dict, contents_dict)

    def test_qml_file_must_be_created_with_specified_contents(self):
        fake_application = fixture_setup.FakeApplication(
            qml_file_contents='Test')
        self.useFixture(fake_application)

        with open(fake_application.qml_file_path, 'r+t') as qml_file:
            qml_file_contents = qml_file.read()

        self.assertEqual('Test', qml_file_contents)

    def test_qml_file_with_default_contents(self):
        fake_application = fixture_setup.FakeApplication()
        self.useFixture(fake_application)

        with open(fake_application.qml_file_path, 'r+t') as qml_file:
            qml_file_contents = qml_file.read()

        self.assertEqual(
            fixture_setup.DEFAULT_QML_FILE_CONTENTS, qml_file_contents)

    def test_desktop_file_must_be_created_with_specified_values(self):
        test_desktop_file_dict = {
            'test key 1': 'test value 1',
            'test key 2': 'test value 2',
            'Exec': 'test',
        }
        fake_application = fixture_setup.FakeApplication(
            desktop_file_dict=test_desktop_file_dict)
        self.useFixture(fake_application)

        with open(fake_application.desktop_file_path, 'r+t') as desktop_file:
            desktop_file_contents = desktop_file.read()

        self.assert_desktop_file_contents(
            desktop_file_contents, test_desktop_file_dict)

    def test_desktop_file_with_launch_command(self):
        test_desktop_file_dict = {'Exec': '{launcher} application'}

        qmlscene = 'lomiriuitoolkit.base.get_toolkit_launcher_command'
        with mock.patch(qmlscene) as mock_qmlscene:
            mock_qmlscene.return_value = 'test_qmlscene_command'
            fake_application = fixture_setup.FakeApplication(
                desktop_file_dict=test_desktop_file_dict)
            self.useFixture(fake_application)

        with open(fake_application.desktop_file_path, 'r+t') as desktop_file:
            desktop_file_contents = desktop_file.read()

        self.assertThat(
            desktop_file_contents,
            Contains('Exec=test_qmlscene_command application'))

    def test_desktop_file_with_qml_file_path(self):
        test_desktop_file_dict = {'Exec': 'qmlscene {qml_file_path}'}

        fake_application = fixture_setup.FakeApplication(
            desktop_file_dict=test_desktop_file_dict)
        self.useFixture(fake_application)

        with open(fake_application.desktop_file_path, 'r+t') as desktop_file:
            desktop_file_contents = desktop_file.read()

        self.assertThat(
            desktop_file_contents,
            Contains(
                'Exec=qmlscene {}'.format(fake_application.qml_file_path)))

    def test_desktop_file_with_default_contents(self):
        get_launcher = 'lomiriuitoolkit.base.get_toolkit_launcher_command'
        with mock.patch(get_launcher) as mock_launcher:
            mock_launcher.return_value = 'testlauncher'
            fake_application = fixture_setup.FakeApplication()
            self.useFixture(fake_application)

        with open(fake_application.desktop_file_path, 'r+t') as desktop_file:
            desktop_file_contents = desktop_file.read()

        expected_desktop_file_dict = {
            'Type': 'Application',
            'Name': 'test',
            'Icon': 'Not important',
            'Exec': 'testlauncher {}'.format(fake_application.qml_file_path),
        }
        self.assert_desktop_file_contents(
            desktop_file_contents, expected_desktop_file_dict)

    def test_desktop_file_must_be_created_in_local_directory(self):
        fake_application = fixture_setup.FakeApplication()
        self.useFixture(fake_application)

        expected_desktop_file_directory = os.path.join(
            os.environ.get('HOME'), '.local', 'share', 'applications')
        self.assertEqual(
            expected_desktop_file_directory,
            os.path.dirname(fake_application.desktop_file_path))

    def test_fake_application_files_must_be_removed_after_test(self):
        fake_application = fixture_setup.FakeApplication(
            url_dispatcher_protocols=['testprotocol'])

        def inner_test():
            class TestWithFakeApplication(testtools.TestCase):
                def test_it(self):
                    self.useFixture(fake_application)
            return TestWithFakeApplication('test_it')

        inner_test().run()
        self.assertThat(fake_application.qml_file_path, Not(FileExists()))
        self.assertThat(fake_application.desktop_file_path, Not(FileExists()))
        self.assertThat(
            fake_application.url_dispatcher_file_path, Not(FileExists()))


class LaunchFakeApplicationTestCase(autopilot_testcase.AutopilotTestCase):

    def test_launch_fake_application_with_qmlscene(self):
        fake_application = fixture_setup.FakeApplication()
        self.useFixture(fake_application)

        application = self.launch_test_application(
            base.get_qmlscene_launch_command(),
            fake_application.qml_file_path,
            '--desktop_file_hint={0}'.format(
                fake_application.desktop_file_path),
            app_type='qt')

        # We can select a component from the application.
        application.select_single('Label', objectName='testLabel')

    def test_launch_fake_application_from_url_dispatcher(self):
        if platform.model() == 'Desktop':
            self.skipTest('Not yet working on desktop')

        path = os.path.abspath(__file__)
        dir_path = os.path.dirname(path)
        qml_file_path = os.path.join(
            dir_path, 'test_fixture_setup.LaunchFakeApplicationTestCase.qml')
        with open(qml_file_path) as qml_file:
            qml_file_contents = qml_file.read()

        fake_application = fixture_setup.FakeApplication(
            qml_file_contents=qml_file_contents,
            url_dispatcher_protocols=['testprotocol'])
        self.useFixture(fake_application)

        self.addCleanup(
            subprocess.check_output,
            ['lomiri-app-stop', fake_application.application_name])

        subprocess.check_output(
            ['url-dispatcher', 'testprotocol://test'])

        pid = int(subprocess.check_output(
            ['lomiri-app-pid', fake_application.application_name]).strip())
        self.assertGreater(pid, 0)


class FakeHomeTestCase(testtools.TestCase):

    def test_fake_home_fixture_patches_env_var(self):
        original_home = os.environ.get('HOME')
        fake_home = tempfile.gettempdir()
        result = testtools.TestResult()

        def inner_test():
            class TestWithFakeHome(testtools.TestCase):
                def test_it(self):
                    fake_home_fixture = fixture_setup.FakeHome(fake_home)
                    fake_home_fixture.should_copy_xauthority_file = False
                    self.useFixture(fake_home_fixture)
                    self.assertEqual(
                        fake_home, os.environ.get('HOME'))
            return TestWithFakeHome('test_it')

        inner_test().run(result)

        self.assertTrue(
            result.wasSuccessful(),
            'Failed to fake the home: {}'.format(result.failures))
        self.assertEqual(original_home, os.environ.get('HOME'))

    def test_fake_home_fixture_must_create_default_directory(self):
        original_home = os.environ.get('HOME')
        self.useFixture(fixture_setup.FakeHome())

        home_parent_directory, _ = os.path.split(os.environ.get('HOME'))
        self.assertEqual(
            home_parent_directory,
            os.path.join(original_home, 'autopilot', 'fakeenv'))

    def test_fake_home_fixture_must_copy_xauthority(self):
        # Fake the home first so we don't write the xauthority on the real
        # home.
        self.useFixture(fixture_setup.FakeHome())

        open(os.path.join(os.environ.get('HOME'), '.Xauthority'), 'w').close()

        self.useFixture(fixture_setup.FakeHome())
        self.assertTrue(
            os.path.exists(
                os.path.join(os.environ.get('HOME'), '.Xauthority')))


class HideUnity7LauncherTestCase(
        tests.LomiriUIToolkitWithFakeAppRunningTestCase):

    def setUp(self):
        if platform.model() != 'Desktop':
            self.skipTest('Unity 7 runs only on desktop.')
        self.useFixture(fixture_setup.HideUnity7Launcher())
        super().setUp()

    def test_maximized_application_must_use_all_screen_width(self):
        application = self.process_manager.get_running_applications()[0]
        window = application.get_windows()[0]

        # Maximize window.
        screen = display.Display.create()
        screen_width = screen.get_screen_width()
        screen_height = screen.get_screen_height()
        window.resize(screen_width, screen_height)

        def get_window_width():
            _, _, window_width, _ = window.geometry
            return window_width

        self.assertThat(
            get_window_width,
            Eventually(Equals(screen_width)))


class FakeDisplay(object):

    """Fake display with fixed widht and height for use in tests."""

    def __init__(self, width, height):
        super().__init__()
        self.width = width
        self.height = height

    def get_screen_width(self):
        return self.width

    def get_screen_height(self):
        return self.height


class SimulateDeviceTestCase(autopilot_testcase.AutopilotTestCase):

    scenarios = [
        ('Device equal to screen', {
            'device_width': 100, 'device_height': 100, 'device_grid_unit': 20,
            'screen_width': 100, 'screen_height': 100,
            'expected_width': 100, 'expected_height': 100,
            'expected_grid_unit': 20}),
        ('Device smaller than screen', {
            'device_width': 100, 'device_height': 90, 'device_grid_unit': 20,
            'screen_width': 110, 'screen_height': 100,
            'expected_width': 100, 'expected_height': 90,
            'expected_grid_unit': 20}),
        ('Device wider than screen', {
            'device_width': 200, 'device_height': 90, 'device_grid_unit': 20,
            'screen_width': 110, 'screen_height': 100,
            'expected_width': 100, 'expected_height': 45,
            'expected_grid_unit': 10}),
        ('Device taller than screen', {
            'device_width': 100, 'device_height': 180, 'device_grid_unit': 20,
            'screen_width': 110, 'screen_height': 100,
            'expected_width': 50, 'expected_height': 90,
            'expected_grid_unit': 10}),
        ('Device bigger than screen', {
            'device_width': 200, 'device_height': 180, 'device_grid_unit': 20,
            'screen_width': 110, 'screen_height': 100,
            'expected_width': 100, 'expected_height': 90,
            'expected_grid_unit': 10}),
    ]

    def test_simulate_device_fixture_with_size_smaller_than_screen(self):
        """Test the simulation of a device that fits the screen.

        All the attributes of the fixture must remain the same.

        """

        simulate_device_fixture = fixture_setup.SimulateDevice(
            self.device_width, self.device_height, self.device_grid_unit)
        fake_display = FakeDisplay(self.screen_width, self.screen_height)
        simulate_device_fixture._screen = fake_display

        self.useFixture(simulate_device_fixture)

        self.assertEqual(
            int(os.environ.get('GRID_UNIT_PX')), self.expected_grid_unit)
        self.assertEqual(
            simulate_device_fixture.app_width, self.expected_width)
        self.assertEqual(
            simulate_device_fixture.app_height, self.expected_height)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Lomiri UI Toolkit Gallery"""

import os

from autopilot.matchers import Eventually
from testtools.matchers import Equals

import lomiriuitoolkit
from lomiriuitoolkit import fixture_setup


class GalleryTestCase(lomiriuitoolkit.tests.QMLFileAppTestCase):

    """Base class for gallery test cases."""

    local_desktop_file_path = None

    def setUp(self):
        self.test_source_path = self._get_test_source_path()
        self.test_qml_file_path = self._get_test_qml_file_path()

        if self.should_simulate_device():
            # Hide the Unity7 launcher because it takes space that might be
            # needed by the app with the simulated size.
            self.useFixture(fixture_setup.HideUnity7Launcher())
            # This sets the grid units, so it should be called before launching
            # the app.
            self.simulate_device()

        super().setUp()

        if self.should_simulate_device():
            # XXX Currently we have no way to launch the application with a
            # specific size, so we must resize it after it's launched.
            # --elopio - 2014-06-25
            self.resize_window()

    def should_simulate_device(self):
        return (hasattr(self, 'app_width') and hasattr(self, 'app_height') and
                hasattr(self, 'grid_unit_px'))

    def simulate_device(self):
        simulate_device_fixture = self.useFixture(fixture_setup.SimulateDevice(
            self.app_width, self.app_height, self.grid_unit_px))
        self.app_width = simulate_device_fixture.app_width
        self.app_height = simulate_device_fixture.app_height

    def resize_window(self):
        application = self.process_manager.get_running_applications()[0]
        window = application.get_windows()[0]
        window.resize(self.app_width, self.app_height)

        def get_window_size():
            _, _, window_width, window_height = window.geometry
            return window_width, window_height

        self.assertThat(
            get_window_size,
            Eventually(Equals((self.app_width, self.app_height))))

    def _get_test_source_path(self):
        if self._application_source_exists():
            test_source_path = self._get_path_to_gallery_source()
        else:
            test_source_path = self._get_path_to_installed_gallery()
        assert os.path.exists(test_source_path)
        return test_source_path

    def _get_path_to_gallery_source(self):
        return os.path.join(
            lomiriuitoolkit.tests.get_path_to_source_root(), 'examples',
            'lomiri-ui-toolkit-gallery')

    def _application_source_exists(self):
        return 'LOMIRI_UI_TOOLKIT_AUTOPILOT_FROM_SOURCE' in os.environ

    def _get_test_qml_file_path(self):
        return os.path.join(
            self.test_source_path,
            'lomiri-ui-toolkit-gallery.qml')

    def _get_path_to_installed_gallery(self):
        host_multiarch = lomiriuitoolkit.base.get_host_multiarch()
        return '/usr/lib/{}/qt5/examples//lomiri-ui-toolkit/examples/' \
               'lomiri-ui-toolkit-gallery'.format(host_multiarch)

    def open_page(self, page):
        """Open a page of the widget gallery.

        :param page: The objectName of the element in the index list that opens
            the page.

        """
        list_view = self.main_view.select_single(objectName="widgetList")
        element = list_view.click_element(page)
        # Do not check the ListItem's highlight
        # the actual highlight is implemented in the LomiriListView
        self.checkPageHeader(element.text)
        return element

    def tearDown(self):
        super().tearDown()
        # We can't delete the desktop file before we close the application,
        # so we save it on an attribute to be deleted on tear down.
        if self.local_desktop_file_path is not None:
            os.remove(self.local_desktop_file_path)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import logging as autopilot_logging
from autopilot.introspection import dbus
from autopilot import introspection

from lomiriuitoolkit._custom_proxy_objects import _common


logger = logging.getLogger(__name__)


class Dialog(_common.LomiriUIToolkitCustomProxyObjectBase):
    """Autopilot helper for the Dialog component."""


class TextInputPopover(_common.LomiriUIToolkitCustomProxyObjectBase):
    """TextInputPopover Autopilot emulator ."""

    def click_option_by_text(self, text):
        """Click a button on the popover.

        XXX We are receiving the text because there's no way to set the
        objectName on the action. This is reported at
        https://bugs.launchpad.net/lomiri-ui-toolkit/+bug/1205144
        --elopio - 2013-07-25

        :parameter text: The text of the button.
        :raise ToolkitException: If the popover is not open.

        """
        if not self.visible:
            raise _common.ToolkitException('The popover is not open.')
        button = self._get_button(text)
        if button is None:
            raise _common.ToolkitException(
                'Button with text "{0}" not found.'.format(text))
        self.pointing_device.click_object(button)
        if self.autoClose:
            try:
                self.visible.wait_for(False)
            except dbus.StateNotFoundError:
                # The popover was removed from the tree.
                pass

    def _get_button(self, text):
        # Try the C++ class name in case this is a 1.3 AbstractButton
        try:
            return self.wait_select_single(
                'LomiriToolkit::UCAbstractButton', text=text)
        except dbus.StateNotFoundError:
            return self.wait_select_single('AbstractButton', text=text)


class ActionSelectionPopover(_common.LomiriUIToolkitCustomProxyObjectBase):
    """ActionSelectionPopover Autopilot custom proxy object."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        if super().validate_dbus_object(path, state):
            return True

        name = introspection.get_classname_from_path(path)
        if name == b'OverflowPanel':
            return True
        if name == b'ListItemPopover':
            return True

        return False

    def click_action_button(self, action_object_name):
        """Click an action button on the popover.

        :parameter object_name: The QML objectName property of the action
        :raise ToolkitException: If there is no visible button with that object
            name or the popover is not open.

        """

        if not self.visible:
            raise _common.ToolkitException('The popover is not open.')
        try:
            object_name = action_object_name + "_button"
            button = self.select_single(objectName=object_name)
        except dbus.StateNotFoundError:
            raise _common.ToolkitException(
                'Button for action with objectName "{0}" not found.'.format(
                    action_object_name))
        if not button.visible:
            raise _common.ToolkitException(
                'Button for action with objectName "{0}" not visible.'.format(
                    action_object_name))
        self.pointing_device.click_object(button)
        if self.autoClose:
            try:
                self.visible.wait_for(False)
            except dbus.StateNotFoundError:
                # The popover was removed from the tree.
                pass

    def click_button_by_text(self, text):
        """Click a button on the popover.

        :parameter text: The text of the button.
        :raise ToolkitException: If there is no visible button with that label
            or the popover is not open.

        """
        if not self.visible:
            raise _common.ToolkitException('The popover is not open.')
        button = self._get_button(text)
        if button is None:
            raise _common.ToolkitException(
                'Button with text "{0}" not found.'.format(text))
        self.pointing_device.click_object(button)
        if self.autoClose:
            try:
                self.visible.wait_for(False)
            except dbus.StateNotFoundError:
                # The popover was removed from the tree.
                pass

    def _get_button(self, text):
        buttons = self.select_many('Empty')
        for button in buttons:
            if button.text == text:
                return button


class ComposerSheet(_common.LomiriUIToolkitCustomProxyObjectBase):
    """ComposerSheet Autopilot custom proxy object."""

    @autopilot_logging.log_action(logger.info)
    def confirm(self):
        """Confirm the composer sheet."""
        button = self.select_single('Button', objectName='confirmButton')
        self.pointing_device.click_object(button)
        self.wait_until_destroyed()

    @autopilot_logging.log_action(logger.info)
    def cancel(self):
        """Cancel the composer sheet."""
        button = self.select_single('Button', objectName='cancelButton')
        self.pointing_device.click_object(button)
        self.wait_until_destroyed()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from unittest import mock

import testtools

from lomiriuitoolkit import lomiri_scenarios


class ScenariosTestCase(testtools.TestCase):

    def test_get_nexus_4_scenario(self):
        expected_scenarios = [
            ('Simulating Nexus 4 in desktop',
             dict(app_width=768, app_height=1280, grid_unit_px=18)),
        ]
        with mock.patch('autopilot.platform.model') as mock_model:
            mock_model.return_value = 'Desktop'
            scenarios = lomiri_scenarios.get_device_simulation_scenarios(
                devices=lomiri_scenarios.NEXUS4_DEVICE)
        self.assertEqual(expected_scenarios, scenarios)

    def test_get_nexus_10_scenario(self):
        expected_scenarios = [
            ('Simulating Nexus 10 in desktop',
             dict(app_width=2560, app_height=1600, grid_unit_px=20))
        ]

        with mock.patch('autopilot.platform.model') as mock_model:
            mock_model.return_value = 'Desktop'
            scenarios = lomiri_scenarios.get_device_simulation_scenarios(
                devices=lomiri_scenarios.NEXUS10_DEVICE)
        self.assertEqual(expected_scenarios, scenarios)

    def test_get_default_scenarios_must_return_supported_devices(self):
        expected_scenarios = [
            ('Simulating Nexus 4 in desktop',
             dict(app_width=768, app_height=1280, grid_unit_px=18)),
            ('Simulating Nexus 10 in desktop',
             dict(app_width=2560, app_height=1600, grid_unit_px=20))
        ]

        with mock.patch('autopilot.platform.model') as mock_model:
            mock_model.return_value = 'Desktop'
            scenarios = lomiri_scenarios.get_device_simulation_scenarios()
        self.assertEqual(expected_scenarios, scenarios)

    def test_get_scenarios_on_device_must_return_no_simulation_scenarios(self):
        with mock.patch('autopilot.platform.model') as mock_model:
            mock_model.return_value = 'Not Desktop'
            scenarios = lomiri_scenarios.get_device_simulation_scenarios()

        self.assertEqual([('Not Desktop', {})], scenarios)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the Switch Qml component."""

from autopilot.matchers import Eventually
from textwrap import dedent
from testtools.matchers import Is, Not, Equals
from testtools import skip
import os
from tavastia.tests import TavastiaTestCase

class EnabledSwitchTests(TavastiaTestCase):
    """Tests for an enabled Switch component."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"EnabledSwitchTests")

    def test_can_select_switch(self):
        """Must be able to select the Qml Switch component."""

        obj = self.app.select_single('Switch')
        self.assertThat(obj, Not(Is(None)))

    def test_clicked_signal_emitted(self):
        """Clicking the Switch component must emit the clicked() signal."""

        obj = self.app.select_single('Switch')
        signal = obj.watch_signal('clicked(QVariant)')

        self.assertThat(obj.checked, Equals(False))

        self.mouse.move_to_object(obj)
        self.mouse.click()

        self.assertThat(obj.checked, Eventually(Equals(True)))

        self.assertThat(signal.was_emitted, Equals(True))
        self.assertThat(signal.num_emissions, Equals(1))

class DisabledSwitchTests(TavastiaTestCase):
    """Tests for an enabled Switch component."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"DisabledSwitchTests")

    def test_clicked_signal_not_emitted(self):
        """Clicking a disabled switch must not emit the clicked() signal."""

        obj = self.app.select_single('Switch')
        signal = obj.watch_signal('clicked(QVariant)')

        self.assertThat(obj.checked, Equals(False))

        self.mouse.move_to_object(obj)
        self.mouse.click()

        self.assertThat(obj.checked, Eventually(Equals(False)))

        self.assertThat(signal.was_emitted, Equals(False))
        self.assertThat(signal.num_emissions, Equals(0))




# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013-2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
from unittest import mock

import lomiriuitoolkit
from lomiriuitoolkit import tests


class ToolbarTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_toolbar.ToolbarTestCase.qml')

    def setUp(self):
        super().setUp()
        self.toolbar = self.main_view.get_toolbar()
        # toolbar may be opened or closed now, depending on whether
        # the application has been deactivated and resumed already

    def test_open_toolbar(self):
        self.toolbar.open()
        self.assertTrue(self.toolbar.opened)
        self.assertFalse(self.toolbar.animating)

    def test_opened_toolbar_is_not_opened_again(self):
        self.toolbar.open()
        with mock.patch.object(
                self.main_view.pointing_device, 'drag') as mock_drag:
            self.toolbar.open()

        self.assertFalse(mock_drag.called)
        self.assertTrue(self.toolbar.opened)

    def test_close_toolbar(self):
        self.toolbar.open()
        self.toolbar.close()
        self.assertFalse(self.toolbar.opened)
        self.assertFalse(self.toolbar.animating)

    def test_closed_toolbar_is_not_closed_again(self):
        self.toolbar.close()
        with mock.patch.object(
                self.main_view.pointing_device, 'drag') as mock_drag:
            self.toolbar.close()

        self.assertFalse(mock_drag.called)
        self.assertFalse(self.toolbar.opened)

    def test_click_toolbar_button(self):
        self.toolbar.close()
        label = self.app.select_single('Label', objectName='clicked_label')
        self.assertNotEqual(label.text, 'Button clicked.')
        self.toolbar.open()
        self.toolbar.click_button('buttonName')
        self.assertEqual(label.text, 'Button clicked.')

    def test_click_unexisting_button(self):
        self.main_view.open_toolbar()
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, self.toolbar.click_button,
            'unexisting')
        self.assertEqual(
            str(error), 'Button with objectName "unexisting" not found.')

    def test_click_button_on_closed_toolbar(self):
        self.toolbar.close()
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException, self.toolbar.click_button,
            'buttonName')
        self.assertEqual(
            str(error),
            'Toolbar must be opened before calling click_button().')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from autopilot import platform


NEXUS4_DEVICE = 'Nexus4'
NEXUS10_DEVICE = 'Nexus10'
DEFAULT_DEVICES = (NEXUS4_DEVICE, NEXUS10_DEVICE)


def get_device_simulation_scenarios(devices=DEFAULT_DEVICES):
    """Return a list of devices to be simulated on tests.

    :param devices: The device or devices to simulate. Default value is all the
        officially supported devices.
    :type devices: string or sequence of strings.
    :return: A list of scenarios to be used with the testscenarios python
        module, with the values of app_width, app_height and grid_unit
        corresponding to the selected device.

    """
    if platform.model() == 'Desktop':
        return _get_device_simulation_scenarios_for_desktop(devices)
    else:
        return [(platform.model(), {})]


def _get_device_simulation_scenarios_for_desktop(devices):
    scenarios = []
    if NEXUS4_DEVICE in devices:
        scenarios.append(
            ('Simulating Nexus 4 in desktop',
             dict(app_width=768, app_height=1280, grid_unit_px=18))
        )
    if NEXUS10_DEVICE in devices:
        scenarios.append(
            ('Simulating Nexus 10 in desktop',
             dict(app_width=2560, app_height=1600, grid_unit_px=20))
        )
    return scenarios



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import logging as autopilot_logging
from lomiriuitoolkit._custom_proxy_objects import _common


logger = logging.getLogger(__name__)


class UCListItem(_common.LomiriUIToolkitCustomProxyObjectBase):
    """Base class to emulate swipe for leading and trailing actions."""

    @autopilot_logging.log_action(logger.info)
    def _swipe_in_panel(self, panel_item):
        """ Swipe in panel (leading/trailing)"""
        x, y, width, height = self.globalRect
        if panel_item == 'leading':
            start_x = x + (width * 0.2)
            stop_x = x + (width * 0.8)
        elif panel_item == 'trailing':
            start_x = x + (width * 0.8)
            stop_x = x + (width * 0.2)
        else:
            raise _common.ToolkitException(
                'No {0} panel found in a ListItem'.format(panel_item))
        start_y = stop_y = y + (height // 2)
        self.pointing_device.drag(start_x, start_y, stop_x, stop_y)
        # wait till animation finishes
        self.contentMoving.wait_for(False)

    def _click_on_panel_action(self, panel_item, action_object, wait_function):
        self._swipe_in_panel(panel_item)
        try:
            button_name = 'actionbutton_' + action_object
            action_button = self.select_single(objectName=button_name)
        except:
            raise _common.ToolkitException(
                'The requested action not found on {0} side'.
                format(panel_item))

        self.pointing_device.click_object(action_button)
        if wait_function is None:
            # wait till animation finishes
            self.contentMoving.wait_for(False)
        else:
            wait_function()

    @autopilot_logging.log_action(logger.info)
    def trigger_leading_action(self, action_objectName, wait_function=None):
        """Swipe the item in from left to right to open leading actions
           and click on the button representing the requested action.

           parameters: action_objectName - object name of the action to be
                       triggered.
                       wait_function - a custom wait function to wait till the
                       action is triggered
        """
        self._click_on_panel_action('leading', action_objectName,
                                    wait_function)

    @autopilot_logging.log_action(logger.info)
    def trigger_trailing_action(self, action_objectName, wait_function=None):
        """Swipe the item in from right to left to open trailing actions
           and click on the button representing the requested action.

           parameters: action_objectName - object name of the action to be
                       triggered.
                       wait_function - a custom wait function to wait till the
                       action is triggered
        """
        self._click_on_panel_action('trailing', action_objectName,
                                    wait_function)

    @autopilot_logging.log_action(logger.info)
    def toggle_selected(self):
        """Toggles selected state of the ListItem. """
        toggle = self.select_single(objectName='listitem_select')
        self.pointing_device.click_object(toggle)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012-2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Lomiri UI Toolkit autopilot tests."""

import fixtures
import os
import tempfile

from autopilot.input import Pointer
from autopilot.matchers import Eventually
from testtools.matchers import Is, Not, Equals

import lomiriuitoolkit
from lomiriuitoolkit import base, fixture_setup


_DESKTOP_FILE_CONTENTS = ("""[Desktop Entry]
Type=Application
Exec=/usr/bin/whoami
Path=Not important
Name=Test app
Icon=Not important
""")


def _write_test_desktop_file():
    desktop_file_dir = get_local_desktop_file_directory()
    if not os.path.exists(desktop_file_dir):
        os.makedirs(desktop_file_dir)
    # Strip underscores which look like an APP_ID to Lomiri
    # See https://bugs.launchpad.net/lomiri-ui-toolkit/+bug/1329141
    chars = tempfile._RandomNameSequence.characters.strip("_")
    tempfile._RandomNameSequence.characters = chars
    desktop_file = tempfile.NamedTemporaryFile(
        suffix='.desktop', dir=desktop_file_dir, delete=False)
    desktop_file.write(_DESKTOP_FILE_CONTENTS.encode('utf-8'))
    desktop_file.close()
    return desktop_file.name


def get_local_desktop_file_directory():
    return os.path.join(os.environ['HOME'], '.local', 'share', 'applications')


def _get_module_include_path():
    return os.path.join(get_path_to_build_root(), 'modules')


def get_path_to_build_root():
    if "UITK_BUILD_ROOT" in os.environ:
        return os.environ["UITK_BUILD_ROOT"]
    return _guess_root_path()


def get_path_to_source_root():
    if "UITK_SOURCE_ROOT" in os.environ:
        return os.environ["UITK_SOURCE_ROOT"]
    return _guess_root_path()


def _guess_root_path():
    return os.path.abspath(
        os.path.join(
            os.path.dirname(__file__), '..', '..', '..', '..'))


class LomiriUIToolkitWithFakeAppRunningTestCase(
        base.LomiriUIToolkitAppTestCase):

    test_qml = ("""
import QtQuick 2.0
import Lomiri.Components 1.1

MainView {
    width: units.gu(48)
    height: units.gu(80)
    objectName: "mainView"
}
""")

    def setUp(self):
        super().setUp()
        self.launch_application()

    def launch_application(self):
        fake_application = fixture_setup.FakeApplication(
            qml_file_contents=self.test_qml)
        self.useFixture(fake_application)

        local_modules_path = _get_module_include_path()
        if os.path.exists(local_modules_path):
            self.use_local_modules(local_modules_path)
        desktop_file_name = os.path.basename(
            fake_application.desktop_file_path)
        application_name, _ = os.path.splitext(desktop_file_name)
        self.app = self.launch_test_application(application_name)

    def use_local_modules(self, local_modules_path):
        env_vars = [
            'QML_IMPORT_PATH',
            'QML2_IMPORT_PATH',
            'LOMIRI_UI_TOOLKIT_THEMES_PATH'
        ]
        for env in env_vars:
            self.useFixture(fixtures.EnvironmentVariable(env,
                                                         local_modules_path))


class QMLStringAppTestCase(LomiriUIToolkitWithFakeAppRunningTestCase):
    """Base test case for self tests that define the QML on an string."""

    def setUp(self):
        super().setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

        self.pointing_device = Pointer(self.input_device_class.create())

    @property
    def main_view(self):
        """
        Return the MainView of the app, selected by objectName.

        QML code used for testing must define the objectName
        of the MainView to be 'mainView'.
        """
        return self.app.select_single(objectName='mainView')


class FlickDirection:
    """Enum for flick or scroll direction."""

    UP, DOWN, LEFT, RIGHT = range(0, 4)


class QMLFileAppTestCase(base.LomiriUIToolkitAppTestCase):
    """Base test case for self tests that launch a QML file."""

    test_qml_file_path = '/path/to/file.qml'
    desktop_file_path = None

    def setUp(self):
        super().setUp()
        self.pointing_device = Pointer(self.input_device_class.create())
        self.launch_application()

    def get_command_line(self, command_line):
        return command_line

    def launch_application(self):
        desktop_file_path = self._get_desktop_file_path()
        command_line = [
            base.get_toolkit_launcher_command(),
            "-I", _get_module_include_path(),
            self.test_qml_file_path,
            '--desktop_file_hint={0}'.format(desktop_file_path)
            ]
        self.app = self.launch_test_application(
            *self.get_command_line(command_line),
            emulator_base=lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase,
            app_type='qt')

        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

    def _get_desktop_file_path(self):
        if self.desktop_file_path is None:
            desktop_file_path = _write_test_desktop_file()
            self.addCleanup(os.remove, desktop_file_path)
            return desktop_file_path
        else:
            self.desktop_file_path

    @property
    def main_view(self):
        """
        Return the MainView of the app, selected by objectName.

        QML code used for testing must define the objectName
        of the MainView to be 'mainView'.
        """
        return self.app.select_single(objectName='mainView')

    def getOrientationHelper(self):
        orientationHelper = self.main_view.select_many(
            "OrientationHelper")[0]
        self.assertThat(orientationHelper, Not(Is(None)))
        return orientationHelper

    def checkPageHeader(self, pageTitle):
        header_label = self.main_view.wait_select_single(
            objectName="header_title_label", text=pageTitle, visible=True)
        self.assertThat(header_label, Not(Is(None)))
        self.assertThat(header_label.visible, Eventually(Equals(True)))

    def getObject(self, objectName):
        obj = self.app.select_single(objectName=objectName)
        self.assertThat(obj, Not(Is(None)))
        return obj

    def tap(self, objectName):
        obj = self.getObject(objectName)
        self.pointing_device.move_to_object(obj)
        self.pointing_device.click()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import logging as autopilot_logging
from autopilot.introspection import dbus

from lomiriuitoolkit._custom_proxy_objects import _common


logger = logging.getLogger(__name__)


class Toolbar(_common.LomiriUIToolkitCustomProxyObjectBase):
    """Toolbar Autopilot custom proxy object."""

    @autopilot_logging.log_action(logger.info)
    def open(self):
        """Open the toolbar if it's not already opened.

        :return: The toolbar.

        """
        self.animating.wait_for(False)
        if not self.opened:
            self._drag_to_open()
            self.opened.wait_for(True)
            self.animating.wait_for(False)

        return self

    def _drag_to_open(self):
        x, y, _, _ = self.globalRect
        line_x = x + self.width * 0.50
        start_y = y + self.height - 1
        stop_y = y

        self.pointing_device.drag(line_x, start_y, line_x, stop_y)

    @autopilot_logging.log_action(logger.info)
    def close(self):
        """Close the toolbar if it's opened."""
        self.animating.wait_for(False)
        if self.opened:
            self._drag_to_close()
            self.opened.wait_for(False)
            self.animating.wait_for(False)

    def _drag_to_close(self):
        x, y, _, _ = self.globalRect
        line_x = x + self.width * 0.50
        start_y = y
        stop_y = y + self.height - 1

        self.pointing_device.drag(line_x, start_y, line_x, stop_y)

    @autopilot_logging.log_action(logger.info)
    def click_button(self, object_name):
        """Click a button of the toolbar.

        The toolbar should be opened before clicking the button, or an
        exception will be raised. If the toolbar is closed for some reason
        (e.g., timer finishes) after moving the mouse cursor and before
        clicking the button, it is re-opened automatically by this function.

        :parameter object_name: The QML objectName property of the button.
        :raise ToolkitException: If there is no button with that object
            name.

        """
        # ensure the toolbar is open
        if not self.opened:
            raise _common.ToolkitException(
                'Toolbar must be opened before calling click_button().')
        try:
            button = self._get_button(object_name)
        except dbus.StateNotFoundError:
            raise _common.ToolkitException(
                'Button with objectName "{0}" not found.'.format(object_name))
        self.pointing_device.move_to_object(button)
        # ensure the toolbar is still open (may have closed due to timeout)
        self.open()
        # click the button
        self.pointing_device.click_object(button)

    def _get_button(self, object_name):
        return self.select_single(objectName=object_name)

    @autopilot_logging.log_action(logger.info)
    def click_back_button(self):
        """Click the back button of the toolbar."""
        self.click_button('back_toolbar_button')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import time

from autopilot.introspection import dbus

from lomiriuitoolkit._custom_proxy_objects import _common


class OptionSelector(_common.LomiriUIToolkitCustomProxyObjectBase):
    """OptionSelector Autopilot custom proxy object"""

    def get_option_count(self):
        """Gets the number of items in the option selector"""
        self.list_view = self.select_single("QQuickListView")
        return self.list_view.count

    def get_selected_index(self):
        """Gets the current selected index of the QQuickListView"""
        self.list_view = self.select_single("QQuickListView")
        return self.list_view.currentIndex

    def get_selected_text(self):
        """gets the text of the currently selected item"""
        option_selector_delegate = self.select_single(
            'OptionSelectorDelegate', focus='True')
        current_label = option_selector_delegate.select_single(
            'UCLabel', visible='True')
        return current_label.text

    def get_current_label(self):
        """gets the text of the currently selected item"""
        option_selector_delegate = self.select_single(
            'OptionSelectorDelegate', focus='True')
        current_label = option_selector_delegate.select_single(
            'UCLabel', visible='True')
        return current_label

    def _expand(self):
        """Expand an optionselector if it's collapsed"""
        # if just collapsed it can think that the item is expanded
        #  https://bugs.launchpad.net/lomiri-ui-toolkit/+bug/1240288
        time.sleep(1)
        if not self.expanded and not self.currentlyExpanded:
            self.pointing_device.click_object(self.get_current_label())
            self.currentlyExpanded.wait_for(True)
            # selecting the same item too quickly after expand
            # causes the wrong item to be selected
            # https://bugs.launchpad.net/lomiri-ui-toolkit/+bug/1231939
            time.sleep(1)

    def select_option(self, *args, **kwargs):
        """Select delegate in option selector

        Example usage::
            select_option(objectName="myOptionSelectorDelegate")
            select_option('UCLabel', text="some_text_here")

        :parameter kwargs: keywords used to find property(s) of delegate in
            option selector

        """

        if args:
            try:
                select_object = self.select_single(
                    *args,
                    **kwargs
                )
            except dbus.StateNotFoundError:
                raise _common.ToolkitException(
                    'OptionSelectorDelegate with args {} and kwargs {} not '
                    'found'.format(args, kwargs)
                )

        else:
            try:
                select_object = self.select_single(
                    'OptionSelectorDelegate',
                    **kwargs
                )
            except dbus.StateNotFoundError:
                raise _common.ToolkitException(
                    'OptionSelectorDelegate with kwargs {} not found'.format(
                        kwargs)
                )

        self._expand()
        self.pointing_device.click_object(select_object)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Lomiri UI Toolkit Autopilot tests and helpers."""

from lomiriuitoolkit import (
    base,
    fixture_setup,
    tests,
    lomiri_scenarios
)
from lomiriuitoolkit._custom_proxy_objects import (
    ActionBar,
    AppHeader,
    check_autopilot_version,
    CheckBox,
    get_keyboard,
    get_pointing_device,
    Header,
    Dialog,
    listitems,
    UCListItem,
    MainView,
    OptionSelector,
    pickers,
    popups,
    QQuickFlickable,
    QQuickGridView,
    QQuickListView,
    Sections,
    TabBar,
    Tabs,
    TextArea,
    TextField,
    Toolbar,
    ToolkitException,
    LomiriUIToolkitCustomProxyObjectBase,
    LomiriListView11,
    LomiriListView,
)

__all__ = [
    'AppHeader',
    'base',
    'ActionBar',
    'check_autopilot_version',
    'CheckBox',
    'fixture_setup',
    'get_keyboard',
    'get_pointing_device',
    'Header',
    'Dialog',
    'listitems',
    'UCListItem',
    'MainView',
    'OptionSelector',
    'pickers',
    'popups',
    'QQuickFlickable',
    'QQuickGridView',
    'QQuickListView',
    'Sections',
    'TabBar',
    'Tabs',
    'tests',
    'TextArea',
    'TextField',
    'Toolbar',
    'ToolkitException',
    'lomiri_scenarios',
    'LomiriListView11',
    'LomiriListView',
    'LomiriUIToolkitCustomProxyObjectBase',
]



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Lomiri UI Toolkit Gallery - TextInput components"""

import testscenarios

from lomiriuitoolkit import emulators, lomiri_scenarios
from lomiriuitoolkit.tests.gallery import GalleryTestCase
import locale


class WriteAndClearTextInputTestCase(GalleryTestCase):

    def text_to_write_string():
        return 'Hello World'

    def text_to_write_number():
        return locale.format('%.2f', -1001.23)

    # text_to_write is a function to ensure
    # that locale is evaluated after setUp
    text_input_scenarios = [
        ('standard textfield', dict(
            objectName='textfield_standard',
            text_to_write=text_to_write_string,
            expected_text=text_to_write_string())),
        ('password textfield', dict(
            objectName='textfield_password',
            text_to_write=text_to_write_string,
            expected_text=text_to_write_string())),
        # The text_to_write contains a decimal separator based on locale
        # eg. -1001.23 or -1001,23 or -۱۰۰۱٫۲۳
        # The test expects integers, TextField rejects that character
        ('only integers textfield', dict(
            objectName='textfield_numbers',
            text_to_write=text_to_write_number,
            expected_text='-100123'))
    ]

    scenarios = testscenarios.multiply_scenarios(
        lomiri_scenarios.get_device_simulation_scenarios(),
        text_input_scenarios)

    def setUp(self):
        super().setUp()
        # Apply the user locale from the environment
        # The UITK does the same, so the test must be localized
        locale.setlocale(locale.LC_ALL, "")
        self.open_page('textinputsElement')

    def test_write_on_textfield_must_update_text(self):
        textfield = self.main_view.select_single(
            emulators.TextField, objectName=self.objectName)

        textfield.write(self.text_to_write())
        self.assertEqual(self.expected_text, textfield.text)

    def test_clear_textfield_must_remove_text(self):
        textfield = self.main_view.select_single(
            emulators.TextField, objectName=self.objectName)
        textfield.write(self.text_to_write())

        textfield.clear()
        self.assertEqual('', textfield.text)


class DisabledTextInputTestCase(GalleryTestCase):

    scenarios = lomiri_scenarios.get_device_simulation_scenarios()

    def setUp(self):
        super().setUp()
        self.open_page('textinputsElement')

    def test_textfield_disabled(self):
        textfield_disabled = self.main_view.select_single(
            emulators.TextField, objectName='textfield_disabled')
        self.assertFalse(textfield_disabled.enabled)

        disabled_text = textfield_disabled.text
        self.assertRaises(AssertionError,
                          textfield_disabled.write,
                          'This should not be written')
        self.assertEqual(disabled_text, textfield_disabled.text)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Lomiri UI Toolkit Gallery - OptionSelector component."""

import lomiriuitoolkit
from lomiriuitoolkit import lomiri_scenarios
from lomiriuitoolkit.tests.gallery import GalleryTestCase


class OptionSelectorTestCase(GalleryTestCase):

    scenarios = lomiri_scenarios.get_device_simulation_scenarios()

    def setUp(self):
        super().setUp()
        self.open_page('optionSelectorsElement')

    def test_select_option_from_collapsed_optionselector(self):
        collapsed_option_selector = self.main_view.select_single(
            lomiriuitoolkit.OptionSelector,
            objectName='optionselector_collapsed')

        self.assertEqual(
            collapsed_option_selector.get_selected_text(), 'Value 1')

        collapsed_option_selector.select_option(
            'UCLabel', text='Once upon a time there was a story nobody told.')
        self.assertEqual(collapsed_option_selector.get_selected_index(), 3)

    def test_select_option_from_expanded_optionselector(self):
        expanded_option_selector = self.main_view.select_single(
            lomiriuitoolkit.OptionSelector,
            objectName='optionselector_expanded')

        self.assertEqual(
            expanded_option_selector.get_selected_text(), 'Value 1')

        expanded_option_selector.select_option(
            'UCLabel', text='Once upon a time there was a story nobody told.')
        self.assertEqual(expanded_option_selector.get_selected_index(), 3)

    def test_select_option_from_optionselector_with_custom_model(self):
        option_selector_with_custom_model = self.main_view.select_single(
            lomiriuitoolkit.OptionSelector,
            objectName='optionselector_custommodel')
        option_selector_with_custom_model.swipe_into_view()

        self.assertEqual(
            option_selector_with_custom_model.get_selected_index(), 0)

        option_selector_with_custom_model.select_option(
            'UCLabel', text='Name 4')
        self.assertEqual(
            option_selector_with_custom_model.get_selected_index(), 3)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import unittest

import autopilot
from autopilot import platform, input
from unittest import mock

import lomiriuitoolkit
from lomiriuitoolkit import tests


class CheckAutopilotVersionTestCase(unittest.TestCase):

    def test_lower_version_should_raise_exception(self):
        with mock.patch.object(autopilot, 'version', '1.3'):
            self.assertRaises(
                lomiriuitoolkit.ToolkitException,
                lomiriuitoolkit.check_autopilot_version)

    def test_required_version_should_succeed(self):
        with mock.patch.object(autopilot, 'version', '1.4'):
            lomiriuitoolkit.check_autopilot_version()

    def test_higher_version_should_succeed(self):
        with mock.patch.object(autopilot, 'version', '1.5'):
            lomiriuitoolkit.check_autopilot_version()


# FIXME: this class is not used anywhere, and QMLStringAppTestCase subclasses
#   no longer work (see bug 1578319), so the code below can be deleted.
class LomiriUIToolkitCustomProxyObjectBaseTestCase(tests.QMLStringAppTestCase):

    def test_pointing_device(self):
        self.assertIsInstance(self.app.pointing_device, input.Pointer)

    @unittest.skipIf(platform.model() != 'Desktop', 'Desktop only')
    def test_pointing_device_in_desktop(self):
        self.assertIsInstance(self.app.pointing_device._device, input.Mouse)

    @unittest.skipIf(platform.model() == 'Desktop', 'Phablet only')
    def test_pointing_device_in_phablet(self):
        self.assertIsInstance(self.app.pointing_device._device, input.Touch)

    def test_custom_proxy_objects_should_check_version_on_init(self):
        check_name = (
            'lomiriuitoolkit._custom_proxy_objects._common.'
            'check_autopilot_version')
        with mock.patch(check_name, autospec=True) as mock_check:
            # Instantiate any custom proxy object.
            self.main_view

        mock_check.assert_called_once_with()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import lomiriuitoolkit
from lomiriuitoolkit import tests


class ActionBarTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    overflow_test_qml_file_path = os.path.join(
        dir_path, 'test_actionbar.ActionBarTestCase.qml')
    scrolling_test_qml_file_path = os.path.join(
        dir_path, 'test_actionbar.ScrollingActionBarTestCase.qml')

    scenarios = [
        ('overflow', dict(test_qml_file_path=overflow_test_qml_file_path)),
        ('scrolling', dict(test_qml_file_path=scrolling_test_qml_file_path))
    ]

    def setUp(self):
        super().setUp()
        self.actionbar = self.app.select_single(
            'ActionBar', objectName='ActionBar')
        self.label = self.app.select_single(objectName='Label')
        self.assertEqual(self.label.text, 'No action triggered.')

    def test_custom_proxy_object(self):
        self.assertIsInstance(self.actionbar, lomiriuitoolkit.ActionBar)
        self.assertTrue(self.actionbar.visible)

    def test_click_action_button(self):
        for i in range(1, 4):
            self.actionbar.click_action_button('Action' + str(i))
            self.assertEqual(self.label.text, 'Action ' + str(i) +
                             ' triggered.')

    def test_click_overflow_action_button(self):
        # Action1-3 are directly the ActionBar, Action4-5 are in the overflow
        for i in range(4, 6):
            self.actionbar.click_action_button('Action' + str(i))
            self.assertEqual(self.label.text, 'Action ' + str(i) +
                             ' triggered.')

    def test_click_unexisting_action_button(self):
        error = self.assertRaises(
            lomiriuitoolkit.ToolkitException,
            self.actionbar.click_action_button, 'unexisting')
        self.assertEqual(
            str(error),
            'Button not found')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging

from autopilot import logging as autopilot_logging

from lomiriuitoolkit._custom_proxy_objects import _common


logger = logging.getLogger(__name__)


class CheckBox(_common.LomiriUIToolkitCustomProxyObjectBase):
    """CheckBox Autopilot custom proxy object."""

    @autopilot_logging.log_action(logger.info)
    def check(self, timeout=10):
        """Check a CheckBox, if its not already checked.

        :parameter timeout: number of seconds to wait for the CheckBox to be
            checked. Default is 10.

        """
        if not self.checked:
            self.change_state(timeout)

    @autopilot_logging.log_action(logger.info)
    def uncheck(self, timeout=10):
        """Uncheck a CheckBox, if its not already unchecked.

        :parameter timeout: number of seconds to wait for the CheckBox to be
            unchecked. Default is 10.

        """
        if self.checked:
            self.change_state(timeout)

    @autopilot_logging.log_action(logger.info)
    def change_state(self, timeout=10):
        """Change the state of a CheckBox.

        If it is checked, it will be unchecked. If it is unchecked, it will be
        checked.

        :parameter time_out: number of seconds to wait for the CheckBox state
            to change. Default is 10.

        """
        original_state = self.checked
        self.pointing_device.click_object(self)
        self.checked.wait_for(not original_state, timeout)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013-2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
from unittest import mock

import lomiriuitoolkit
from lomiriuitoolkit import tests


class TextFieldTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    qml_file_path_1_0 = os.path.join(
        dir_path, 'test_textfield.1.0.qml')
    qml_file_path_1_3 = os.path.join(
        dir_path, 'test_textfield.1.3.qml')
    scenarios = [
        ('1.0',
            dict(test_qml_file_path=qml_file_path_1_0)),
        ('1.3',
            dict(test_qml_file_path=qml_file_path_1_3)),
    ]

    def setUp(self):
        super().setUp()
        self.simple_text_field = self.main_view.select_single(
            lomiriuitoolkit.TextField, objectName='simple_text_field')

    def test_text_field_custom_proxy_object(self):
        self.assertIsInstance(
            self.simple_text_field, lomiriuitoolkit.TextField)

    def test_write(self):
        self.simple_text_field.write('test')
        self.assertEqual(self.simple_text_field.text, 'test')

    def test_clear_with_clear_button(self):
        self.simple_text_field.write('test')
        self.simple_text_field.clear()
        self.assertEqual(self.simple_text_field.text, '')

    def test_clear_without_clear_button(self):
        text_field = self.main_view.select_single(
            lomiriuitoolkit.TextField,
            objectName='text_field_without_clear_button')
        text_field.write('test')
        text_field.clear()
        self.assertEqual(text_field.text, '')

    def test_clear_and_write(self):
        self.simple_text_field.write('test1')
        self.simple_text_field.write('test2')
        self.assertEqual(self.simple_text_field.text, 'test2')

    def test_write_without_clear(self):
        self.simple_text_field.write('test1')
        self.simple_text_field.write('test2', clear=False)
        self.assertEqual(self.simple_text_field.text, 'test1test2')

    def test_write_without_clear_writes_at_the_end(self):
        self.simple_text_field.write(
            'long text that will fill more than half of the text field.')
        self.simple_text_field.write('append', clear=False)
        self.assertEqual(
            self.simple_text_field.text,
            'long text that will fill more than half of the text field.append')

    def test_is_empty(self):
        self.assertTrue(self.simple_text_field.is_empty())
        self.simple_text_field.write('test')
        self.assertFalse(self.simple_text_field.is_empty())

    def test_select_all_selects_all_text(self):
        self.simple_text_field.write('Text to select.')
        self.simple_text_field._select_all()

        self.assertTrue(self.simple_text_field._is_all_text_selected())

    def test_select_all_when_already_selected_must_do_nothing(self):
        """Test for select all the text when it's already selected."""
        self.simple_text_field.write('Text to select.')
        self.simple_text_field._select_all()
        with mock.patch.object(
                self.simple_text_field, 'pointing_device') as mock_device:
            self.simple_text_field._select_all()

        self.assertFalse(mock_device.called)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the Tabs Qml component."""

from autopilot.matchers import Eventually
from textwrap import dedent
from testtools.matchers import Is, Not, Equals
from testtools import skip
import os
from tavastia.tests import TavastiaTestCase

class TabsTests(TavastiaTestCase):
    """Tests for Tabs component."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"TabsTests")

    def test_can_select_tabs(self):
        """Must be able to select the Qml Tabs component."""

        obj = self.app.select_single('Tabs')
        self.assertThat(obj, Not(Is(None)))
    




# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Lomiri UI Toolkit Header component."""

import os

from lomiriuitoolkit import tests


class DialogScrollTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_popover.qml')

    def test_dialog_small(self):
        button = self.main_view.select_single(objectName='button_small')
        self.pointing_device.click_object(button)
        dialog = self.main_view.select_single(objectName='dialog_small')
        flickable = dialog.select_single('QQuickFlickable')
        # There's nothing to reveal
        self.assertTrue(flickable.atYEnd)

    def test_dialog_scrollable(self):
        button = self.main_view.select_single(objectName='button_huge')
        self.pointing_device.click_object(button)
        dialog = self.main_view.select_single(objectName='dialog_huge')
        flickable = dialog.select_single('QQuickFlickable')
        buttlet = dialog.select_single(objectName='buttlet49')
        # We can scroll
        self.assertFalse(flickable.atYEnd)
        flickable.swipe_child_into_view(buttlet)
        self.assertEqual(flickable.is_child_visible(buttlet), True)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014, 2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import time

from autopilot import platform

import lomiriuitoolkit
from lomiriuitoolkit import tests


class PullToRefreshTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(dir_path, 'test_pull_to_refresh.qml')

    def setUp(self):
        super().setUp()
        self.label = self.main_view.select_single(
            'Label', objectName='refreshedLabel')
        self.assertEqual(self.label.text, 'Not refreshed.')
        self.flickable_with_pull_to_refresh = self.main_view.select_single(
            lomiriuitoolkit.QQuickFlickable,
            objectName='flickableWithPullToRefresh')

    def test_pull_to_refresh_must_refresh_model(self):
        # TODO disabled due bug #1372446
        self.skipTest(
            'Try to reproduce consistently, bug logged in #1372446')
        self.flickable_with_pull_to_refresh.pull_to_refresh()

        self.assertEqual(self.label.text, 'Refreshed.')

    def test_cancel_pull_to_refresh_must_not_refresh_model(self):
        if platform.model() != 'Desktop':
            # TODO remove the skip once bug http://pad.lv/1266601 is fixed.
            self.skipTest(
                'Autopilot is not yet able to do a drag on the devices '
                'without releasing the simulated finger.')
        self.flickable_with_pull_to_refresh._cancel_pull_to_refresh()
        # Sleep for some time to make sure that the list is not being
        # refreshed.
        time.sleep(3)
        self.assertEqual(self.label.text, 'Not refreshed.')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013-2015 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import time
import os

from autopilot import input
from testtools.matchers import GreaterThan, LessThan
from unittest import mock

import lomiriuitoolkit
from lomiriuitoolkit import tests


class ToggleTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    switch_test_qml_file_path = os.path.join(
        dir_path, 'test_checkbox.ToggleTestCase.test_with_switch.qml')
    checkbox_test_qml_file_path = os.path.join(
        dir_path, 'test_checkbox.ToggleTestCase.test_with_checkbox.qml')

    scenarios = [
        ('checkbox', dict(
            test_qml_file_path=checkbox_test_qml_file_path,
            objectName='test_checkbox')),
        ('switch', dict(
            test_qml_file_path=switch_test_qml_file_path,
            objectName='test_switch'))
    ]

    def setUp(self):
        super().setUp()
        self.toggle = self.main_view.select_single(
            lomiriuitoolkit.CheckBox, objectName=self.objectName)
        self.assertFalse(self.toggle.checked)

    def test_toggle_custom_proxy_object(self):
        self.assertIsInstance(self.toggle, lomiriuitoolkit.CheckBox)

    def test_check_toggle(self):
        self.toggle.check()
        self.assertTrue(self.toggle.checked)

    def test_check_toggle_already_checked(self):
        self.toggle.check()
        with mock.patch.object(input.Pointer, 'click_object') as mock_click:
            self.toggle.check()
        self.assertFalse(mock_click.called)

    def test_uncheck_toggle(self):
        self.toggle.check()
        self.toggle.uncheck()
        self.assertFalse(self.toggle.checked)

    def test_uncheck_toggle_already_unchecked(self):
        with mock.patch.object(input.Pointer, 'click_object') as mock_click:
            self.toggle.uncheck()
        self.assertFalse(mock_click.called)

    def test_change_state_from_checked(self):
        self.toggle.check()
        self.toggle.change_state()
        self.assertFalse(self.toggle.checked)

    def test_change_state_from_unchecked(self):
        self.toggle.change_state()
        self.assertTrue(self.toggle.checked)

    def test_check_with_timeout(self):
        with mock.patch.object(
                lomiriuitoolkit.CheckBox, 'change_state') as mock_change:
            self.toggle.check(timeout=1)

        mock_change.assert_called_once_with(1)

    def test_uncheck_with_timeout(self):
        self.toggle.check()
        with mock.patch.object(
                lomiriuitoolkit.CheckBox, 'change_state') as mock_change:
            self.toggle.uncheck(timeout=1)

        mock_change.assert_called_once_with(1)

    def test_change_state_with_timeout(self):
        with mock.patch.object(self.toggle, 'pointing_device'):
            # mock the pointing device so the checkbox is not clicked.
            timestamp_before_call = time.time()
            self.assertRaises(AssertionError, self.toggle.change_state, 1)

        waiting_time = time.time() - timestamp_before_call
        self.assertThat(waiting_time, GreaterThan(1))
        self.assertThat(waiting_time, LessThan(2))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2016 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from unittest import mock

import os
import testtools
import lomiriuitoolkit
from lomiriuitoolkit import (
    tests,
    units,
)
from lomiriuitoolkit._custom_proxy_objects import _flickable


class FlickableTestCase(testtools.TestCase):

    def test_is_flickable_with_flicking_property_must_return_true(self):
        """is_flickable returns True if flickable property exists."""
        dummy_id = (0, 0)
        dummy_flicking = (0, 'dummy')
        state_with_flicking = {'id': dummy_id, 'flicking': dummy_flicking}
        element = lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase(
            state_with_flicking, '/dummy'.encode(), 'dummy')

        with element.no_automatic_refreshing():
            self.assertTrue(element.is_flickable())

    def test_is_flickable_without_flicking_property_must_return_false(self):
        """is_flickable returns False if flickable property doesn't exist."""
        dummy_id = (0, 0)
        state_without_flicking = {'id': dummy_id}
        element = lomiriuitoolkit.LomiriUIToolkitCustomProxyObjectBase(
            state_without_flicking, '/dummy'.encode(), 'dummy')

        with element.no_automatic_refreshing():
            self.assertFalse(element.is_flickable())


class IsFlickableTestCase(tests.QMLFileAppTestCase):
    """Functional test to check that is_flickable returns the right value.

    We already have tests for is_flickable with mocks, so here we just check
    with some real elements.

    """

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_flickable.IsFlickableTestCase.qml')

    scenarios = [
        ('main view', dict(object_name='mainView', is_flickable=False)),
        ('flickable', dict(object_name='flickable', is_flickable=True)),
        ('list view', dict(object_name='listView', is_flickable=True)),
        ('label', dict(object_name='label', is_flickable=False))
    ]

    def test_is_flickable(self):
        """Test that is_flickable identifies the elements correctly."""
        element = self.app.select_single(objectName=self.object_name)
        self.assertEqual(element.is_flickable(), self.is_flickable)


# FIXME: Only left-to-right (where x=0 is leftmost) layouts are taken into
#   account. Add support for horizontal flicking with right-to-left layouts.
class SwipeFlickableTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_flickable.SwipeFlickableTestCase.qml')

    def setUp(self):
        super().setUp()
        self.flickable = self.main_view.select_single(
            lomiriuitoolkit.QQuickFlickable, objectName='flickable')
        self.label = self.main_view.select_single(objectName='clickedLabel')
        self.assertEqual(self.label.text, 'No element clicked.')

    def test_swipe_into_view_bottom_element(self):
        button = self.main_view.select_single(objectName='bottomButton')
        button.swipe_into_view()

        self.pointing_device.click_object(button)
        self.assertEqual(self.label.text, 'bottomButton')

    def test_swipe_into_view_top_element(self):
        bottomButton = self.main_view.select_single(objectName='bottomButton')
        bottomButton.swipe_into_view()

        topButton = self.main_view.select_single(objectName='topButton')
        topButton.swipe_into_view()

        self.pointing_device.click_object(topButton)
        self.assertEqual(self.label.text, 'topButton')

    def test_swipe_into_view_bottom_right_element(self):
        bottomRightButton = self.main_view.select_single(
            objectName='bottomRightButton')
        bottomRightButton.swipe_into_view()

        self.pointing_device.click_object(bottomRightButton)
        self.assertEqual(self.label.text, 'bottomRightButton')

    def test_swipe_into_view_top_right_element(self):
        topRightButton = self.main_view.select_single(
            objectName='topRightButton')
        topRightButton.swipe_into_view()

        self.pointing_device.click_object(topRightButton)
        self.assertEqual(self.label.text, 'topRightButton')

    def test_swipe_to_top_must_leave_flickable_at_y_beginning(self):
        self.flickable.swipe_to_bottom()
        self.assertFalse(self.flickable.atYBeginning)

        self.flickable.swipe_to_top()
        self.assertTrue(self.flickable.atYBeginning)

    def test_swipe_to_bottom_must_leave_flickable_at_y_end(self):
        self.flickable.swipe_to_top()
        self.assertFalse(self.flickable.atYEnd)

        self.flickable.swipe_to_bottom()
        self.assertTrue(self.flickable.atYEnd)

    def test_swipe_to_leftmost_must_leave_flickable_at_x_beginning(self):
        self.flickable.swipe_to_rightmost()
        self.assertFalse(self.flickable.atXBeginning)

        self.flickable.swipe_to_leftmost()
        self.assertTrue(self.flickable.atXBeginning)

    def test_swipe_to_rightmost_must_leave_flickable_at_x_end(self):
        self.flickable.swipe_to_leftmost()
        self.assertFalse(self.flickable.atXEnd)

        self.flickable.swipe_to_rightmost()
        self.assertTrue(self.flickable.atXEnd)

    def test_swipe_to_show_more_above_with_containers(self):
        """Swipe to show more above must receive containers as parameter."""
        self.flickable.swipe_to_bottom()
        self.assertTrue(self.flickable.atYEnd)

        containers = self.flickable._get_containers()
        self.flickable.swipe_to_show_more_above(containers)
        self.assertFalse(self.flickable.atYEnd)

    def test_swipe_to_show_more_above_without_arguments(self):
        """Calling swipe to show more above must get containers by default."""
        self.flickable.swipe_to_bottom()
        self.assertTrue(self.flickable.atYEnd)

        self.flickable.swipe_to_show_more_above()
        self.assertFalse(self.flickable.atYEnd)

    def test_swipe_to_show_more_below_with_containers(self):
        """Swipe to show more below must receive containers as parameter."""
        self.flickable.swipe_to_top()
        self.assertTrue(self.flickable.atYBeginning)

        containers = self.flickable._get_containers()
        self.flickable.swipe_to_show_more_below(containers)
        self.assertFalse(self.flickable.atYBeginning)

    def test_swipe_to_show_more_below_without_arguments(self):
        """Calling swipe to show more below must get containers by default."""
        self.flickable.swipe_to_top()
        self.assertTrue(self.flickable.atYBeginning)

        self.flickable.swipe_to_show_more_below()
        self.assertFalse(self.flickable.atYBeginning)

    def test_swipe_to_show_more_left_with_containers(self):
        """Swipe to show more left must receive containers as parameter."""
        self.flickable.swipe_to_rightmost()
        self.assertTrue(self.flickable.atXEnd)

        containers = self.flickable._get_containers()
        self.flickable.swipe_to_show_more_left(containers)
        self.assertFalse(self.flickable.atXEnd)

    def test_swipe_to_show_more_left_without_arguments(self):
        """Calling swipe to show more left must get containers by default."""
        self.flickable.swipe_to_rightmost()
        self.assertTrue(self.flickable.atXEnd)

        self.flickable.swipe_to_show_more_left()
        self.assertFalse(self.flickable.atXEnd)

    def test_swipe_to_show_more_right_with_containers(self):
        """Swipe to show more right must receive containers as parameter."""
        self.flickable.swipe_to_leftmost()
        self.assertTrue(self.flickable.atXBeginning)

        containers = self.flickable._get_containers()
        self.flickable.swipe_to_show_more_right(containers)
        self.assertFalse(self.flickable.atXBeginning)

    def test_swipe_to_show_more_right_without_arguments(self):
        """Calling swipe to show more right must get containers by default."""
        self.flickable.swipe_to_leftmost()
        self.assertTrue(self.flickable.atXBeginning)

        self.flickable.swipe_to_show_more_right()
        self.assertFalse(self.flickable.atXBeginning)

    def test_swipe_to_show_more_below_with_bottom_margin(self):
        """Calling swipe to show more below will use the margin in the drag."""
        qquickflickable = self.main_view.select_single(
            lomiriuitoolkit.QQuickFlickable, objectName='flickable')
        qquickflickable.margin_to_swipe_from_bottom = units.gu(6)
        containers = qquickflickable._get_containers()
        bottom = _flickable._get_visible_container_bottom(containers)

        with mock.patch.object(
                qquickflickable.pointing_device, 'drag') as mock_drag:
            try:
                qquickflickable.swipe_to_show_more_below()
            except lomiriuitoolkit.ToolkitException:
                # An exception will be raised because the drag was faked.
                pass

        mock_drag.assert_called_with(
            mock.ANY, bottom - units.gu(6), mock.ANY, mock.ANY, rate=mock.ANY)

    def test_swipe_to_show_more_above_with_top_margin(self):
        """Calling swipe to show more above will use the margin in the drag."""
        qquickflickable = self.main_view.select_single(
            lomiriuitoolkit.QQuickFlickable, objectName='flickable')
        qquickflickable.margin_to_swipe_from_top = units.gu(6)
        containers = qquickflickable._get_containers()
        top = _flickable._get_visible_container_top(containers)

        qquickflickable.swipe_to_bottom()
        with mock.patch.object(
                qquickflickable.pointing_device, 'drag') as mock_drag:
            try:
                qquickflickable.swipe_to_show_more_above()
            except lomiriuitoolkit.ToolkitException:
                # An exception will be raised because the drag was faked.
                pass

        mock_drag.assert_called_with(
            mock.ANY, top + units.gu(6), mock.ANY, mock.ANY, rate=mock.ANY)

    def test_swipe_to_show_more_right_with_right_margin(self):
        """Calling swipe to show more right will use the margin in the drag."""
        qquickflickable = self.main_view.select_single(
            lomiriuitoolkit.QQuickFlickable, objectName='flickable')
        qquickflickable.margin_to_swipe_from_right = units.gu(6)
        containers = qquickflickable._get_containers()
        rightmost = _flickable._get_visible_container_rightmost(containers)

        with mock.patch.object(
                qquickflickable.pointing_device, 'drag') as mock_drag:
            try:
                qquickflickable.swipe_to_show_more_right()
            except lomiriuitoolkit.ToolkitException:
                # An exception will be raised because the drag was faked.
                pass

        mock_drag.assert_called_with(
            rightmost - units.gu(6), mock.ANY, mock.ANY, mock.ANY,
            rate=mock.ANY)

    def test_swipe_to_show_more_left_with_left_margin(self):
        """Calling swipe to show more above will use the margin in the drag."""
        qquickflickable = self.main_view.select_single(
            lomiriuitoolkit.QQuickFlickable, objectName='flickable')
        qquickflickable.margin_to_swipe_from_left = units.gu(6)
        containers = qquickflickable._get_containers()
        leftmost = _flickable._get_visible_container_leftmost(containers)

        qquickflickable.swipe_to_rightmost()
        with mock.patch.object(
                qquickflickable.pointing_device, 'drag') as mock_drag:
            try:
                qquickflickable.swipe_to_show_more_left()
            except lomiriuitoolkit.ToolkitException:
                # An exception will be raised because the drag was faked.
                pass

        mock_drag.assert_called_with(
            leftmost + units.gu(6), mock.ANY, mock.ANY, mock.ANY,
            rate=mock.ANY)

    def test_failed_drag_must_raise_exception(self):
        dummy_coordinates = (0, 0, 10, 10)
        # Patch the pointing device so it does nothing and the swipe fails.
        with mock.patch.object(self.flickable, 'pointing_device'):
            error = self.assertRaises(
                lomiriuitoolkit.ToolkitException,
                self.flickable._slow_drag,
                *dummy_coordinates
            )

        self.assertEqual('Could not swipe in the flickable.', str(error))


class LomiriFlickableTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_flickable.LomiriFlickableTestCase.qml')

    def get_command_line(self, command_line):
        command_line.append('-engine')
        return command_line

    def test_get_lomiri_top_container(self):
        """Test that we can get the top cointainer in Lomiri."""
        # This tests bug http://pad.lv/1314390
        # On Lomiri, the top container is not the first child as it is in all
        # the apps that have a MainView. This makes the first implementation of
        # _get_top_container fail. Instead of going from the top looking for
        # a container, we should start from the flickable until we find the
        # top-most container.
        test_flickable = self.app.select_single(
            lomiriuitoolkit.QQuickFlickable, objectName='testFlickable')
        top_container = test_flickable._get_top_container()
        self.assertIsInstance(top_container, lomiriuitoolkit.MainView)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2012, 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Lomiri UI Toolkit Gallery - LomiriListView component."""

from lomiriuitoolkit.tests.gallery import GalleryTestCase


class LomiriListViewTestCase(GalleryTestCase):

    def _open_page(self):
        super().open_page('lomiriListViewElement')
        self.listView = self.main_view.select_single(
            objectName="lomiriListView")

    def test_pull_to_refresh_enabled(self):
        self._open_page()
        self.assertTrue(self.listView.pull_to_refresh_enabled())

    def test_manual_refresh_wait(self):
        self._open_page()
        self.assertTrue(self.listView.manual_refresh_wait())

    def test_manual_refresh_nowait(self):
        self._open_page()
        self.assertTrue(self.listView.manual_refresh_nowait())
        # wait for completion
        self.listView.wait_refresh_completed()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for the QML PageStack component."""

from autopilot.matchers import Eventually
from textwrap import dedent
from testtools.matchers import Is, Not, Equals
from testtools import skip
import os
from tavastia.tests import TavastiaTestCase

class PageStackTests(TavastiaTestCase):
    """Tests for PageStack component."""

    test_qml_file = "%s/%s.qml" % (os.path.dirname(os.path.realpath(__file__)),"PageStackTests")

    def test_can_select_pagestack(self):
        """Must be able to select the Qml PageStack component."""

        obj = self.app.select_single('PageStack')
        self.assertThat(obj, Not(Is(None)))
    




# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os

import testtools

import lomiriuitoolkit
from lomiriuitoolkit import tests


class QQuickGridViewClassTestCase(testtools.TestCase):

    def test_qquickgridview_custom_proxy_object_inherits_from_flickable(self):
        self.assertTrue(
            issubclass(
                lomiriuitoolkit.QQuickGridView,
                lomiriuitoolkit.QQuickFlickable))


class QQuickGridViewTestCase(tests.QMLFileAppTestCase):

    path = os.path.abspath(__file__)
    dir_path = os.path.dirname(path)
    test_qml_file_path = os.path.join(
        dir_path, 'test_qquickgridview.QQuickGridViewTestCase.qml')

    def test_select_qquickgridview_must_return_custom_proxy_object(self):
        grid_view = self.main_view.select_single(
            lomiriuitoolkit.QQuickGridView, objectName='testListView')
        self.assertIsInstance(grid_view, lomiriuitoolkit.QQuickGridView)
